@@ -296,7 +296,7 @@ PDFFontsDock::PDFFontsDock(PDFDocument *doc)
 	setObjectName("fonts");
 	setWindowTitle(getTitle());
 	table = new QTableWidget(this);
-#ifdef Q_WS_MAC /* don't do this on windows, as the font ends up too small */
+#ifdef Q_OS_MAC /* don't do this on windows, as the font ends up too small */
 	QFont f(table->font());
 	f.setPointSize(f.pointSize() - 2);
 	table->setFont(f);
@@ -575,6 +575,7 @@ PDFScrollArea::PDFScrollArea(QWidget *parent)
        : QAbstractScrollArea(parent),continuous(true),pdf(0), updateWidgetPositionStackWatch(0), onResizeStackWatch(0)
 {
 	viewport()->setBackgroundRole(QPalette::NoRole);
+	viewport()->setAttribute(Qt::WA_AcceptTouchEvents, true);
 	verticalScrollBar()->setSingleStep(20);
 	horizontalScrollBar()->setSingleStep(20);
 	setFocusPolicy(Qt::StrongFocus);
@@ -746,7 +747,8 @@ void PDFScrollArea::updateScrollBars(){
 	if (!continuous) {
 		vbar->setRange(0, v.height() - p.height());
 	} else {
-		vbar->setRange(0,((pdf->pseudoNumPages() + pdf->gridCols() - 1)/ pdf->gridCols()) * pdf->gridRowHeight() - p.height());
+		int totalRows = ((pdf->pseudoNumPages() + pdf->gridCols() - 1)/ pdf->gridCols());
+		vbar->setRange(0, totalRows * pdf->gridRowHeight() - pdf->gridBorder() - p.height() -1);  // -1 is heuristic to prevent activation of the scrollbar in case of fit-page and one-page documents (might be this should be corrected in another place)
 	}
 
 	if (pdf->getScaleOption() == kFitWindow) {
@@ -902,7 +904,6 @@ void PDFClockDock::paintEvent(QPaintEvent * event){
 }
 
 
-#endif
 
 
 MessageFrame::MessageFrame(QWidget *parent) : QFrame(parent), label(0)
@@ -942,3 +943,5 @@ void MessageFrame::showText(const QString &text, QList<QAction *> actions) {
 	}
 	show();
 }
+
+#endif
