@@ -77,7 +77,7 @@ protected:
 private:
 	int oldshape;
 	int page;
-    int overScale;
+	int overScale;
 	qreal	scaleFactor;
 	qreal	parentDpi;
 	QPixmap	image;
@@ -116,7 +116,8 @@ private:
 typedef enum {
 	kFixedMag,
 	kFitWidth,
-	kFitWindow
+	kFitWindow,
+	kFitTextWidth
 } autoScaleOption;
 
 class PDFScrollArea;
@@ -151,6 +152,7 @@ public:
 	Q_INVOKABLE int pageStep();
 	Q_INVOKABLE int gridCols() const;
 	Q_INVOKABLE int gridRowHeight() const;
+	Q_INVOKABLE int gridBorder() const;
 	Q_INVOKABLE PDFDocument * getPDFDocument();
 	Q_INVOKABLE int getPageOffset() const;
 	autoScaleOption getScaleOption() { return scaleOption; }
@@ -168,6 +170,7 @@ public:
 	QRect pageRect(int page) const;
 	QSizeF maxPageSizeF() const;
 	QSizeF gridSizeF(bool ignoreVerticalGrid=false) const;
+	QRectF horizontalTextRangeF() const;
 
 	Q_INVOKABLE void zoom(qreal scale);
 
@@ -183,6 +186,7 @@ protected slots: //not private, so scripts have access
 	void doPageDialog();
 	
 	void fitWidth(bool checked = true);
+	void fitTextWidth(bool checked = true);
 	void zoomIn();
 	void zoomOut();
 	void jumpToSource();
@@ -237,7 +241,7 @@ private:
 	void useMagnifier(const QMouseEvent *inEvent);
 	void goToDestination(const Poppler::LinkDestination& dest);
 	void doLink(const Poppler::Link *link);
-	void annotationClicked(Poppler::Annotation *annotation, const QPointF &scaledPos);
+	void annotationClicked(Poppler::Annotation *annotation, int page);
 	void doZoom(const QPoint& clickPos, int dir, qreal newScaleFactor=1.0);
 
 	PDFScrollArea* getScrollArea();
@@ -300,6 +304,7 @@ private:
 	static QCursor	*zoomOutCursor;
 
 	mutable QSizeF maxPageSize; //cache pageSize
+	mutable QRectF horizontalTextRange;
 
 	QList<int> pageHistory;
 	int pageHistoryIndex;
@@ -330,7 +335,7 @@ class PDFDocument : public QMainWindow, private Ui::PDFDocument
 	Q_PROPERTY(QString fileName READ fileName)
 
 public:
-	explicit PDFDocument(PDFDocumentConfig* const pdfConfig,bool embedded=false);
+    explicit PDFDocument(PDFDocumentConfig* const pdfConfig, bool embedded=false);
 	virtual ~PDFDocument();
 
 	static PDFDocument *findDocument(const QString &fileName);
@@ -381,9 +386,12 @@ protected:
 	virtual void dropEvent(QDropEvent *event);
 	virtual void enterEvent(QEvent *event);
 	virtual void leaveEvent(QEvent *event);
+	virtual void mouseMoveEvent(QMouseEvent *event);
 	void setToolbarsVisible(bool visible);
+	void shortcutOnlyIfFocused(const QList<QAction *> &actions);
 
 public slots:
+	void reloadSettings();
 	void reload(bool fillCache=true);
 	void fillRenderCache(int pg=-1);
 	void sideBySide();
@@ -395,6 +403,9 @@ public slots:
 	void syncFromView(const QString& pdfFile, const QFileInfo& masterFile, int page);
 	void loadFile(const QString &fileName, const QFileInfo& masterFile, bool alert = true);
 	void printPDF();
+	void setAutoHideToolbars(bool enabled);
+	void hideToolbars();
+	void showToolbars();
 private slots:
 	void fileOpen();
 	
@@ -413,7 +424,9 @@ private slots:
 
 	void setGrid();
 
+public slots:
 	void closeSomething();
+private slots:
 	void tileWindows();
 	void stackWindows();
 	void unminimize();
@@ -423,6 +436,10 @@ private slots:
 	void jumpToPage();
 
 	void search(bool backward, bool incremental);
+public:
+	void search(const QString& searchText, bool backward, bool incremental, bool caseSensitive, bool sync);
+	void search();
+private slots:
 	void gotoAnnotation(const PDFAnnotation *ann);
 
 	void zoomFromAction();
@@ -451,6 +468,7 @@ signals:
 
 private:
     void init(bool embedded=false);
+    void setupMenus();
 	void setCurrentFile(const QString &fileName);
 	void loadSyncData();
 
@@ -472,6 +490,17 @@ private:
 	PDFAnnotations * annotations;
 	PDFAnnotationTableView * annotationTable;
 
+	QMenuBar *menubar;
+	QMenu *menuHelp;
+	QMenu *menuFile;
+	QMenu *menuEdit;
+	QMenu *menuView;
+	QMenu *menuGrid;
+	QMenu *menuWindow;
+public:
+	QMenu *menuShow;
+private:
+	QMenu *menuEdit_2;
 
 	QButtonGroup	*toolButtonGroup;
 	QToolButton *comboZoom;
