@@ -10,6 +10,7 @@ struct PDFDocumentConfig{
 	QByteArray windowState;
 
 	int cacheSizeMB;
+	int loadStrategy;
 
 	int dpi;
 	int scaleOption;
@@ -21,7 +22,8 @@ struct PDFDocumentConfig{
 	bool magnifierBorder;
 
 	QString syncFileMask;
-
+	QString highlightColor; // hex rgba, e.g. #FFFF003F
+	int highlightDuration;
 
 //live options
 	int editTool;
