@@ -5,11 +5,11 @@
 
 #include "userquickdialog.h"
 
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 #include "windows.h"
 #endif
 
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 #define ON_WIN(x) x
 #define ON_NIX(x)
 #else
@@ -28,8 +28,8 @@ QString BuildManager::additionalSearchPaths, BuildManager::additionalPdfPaths, B
 #define CMD_DEFINE(up, id) const QString BuildManager::CMD_##up = BuildManager::TXS_CMD_PREFIX + #id;
 CMD_DEFINE(LATEX, latex) CMD_DEFINE(PDFLATEX, pdflatex) CMD_DEFINE(XELATEX, xelatex) CMD_DEFINE(LUALATEX, lualatex) CMD_DEFINE(LATEXMK, latexmk)
 CMD_DEFINE(VIEW_DVI, view-dvi) CMD_DEFINE(VIEW_PS, view-ps) CMD_DEFINE(VIEW_PDF, view-pdf) CMD_DEFINE(VIEW_LOG, view-log)
-CMD_DEFINE(DVIPNG, dvipng) CMD_DEFINE(DVIPS, dvips) CMD_DEFINE(DVIPDF, dvipdf) CMD_DEFINE(PS2PDF, ps2pdf) CMD_DEFINE(GS, gs) CMD_DEFINE(MAKEINDEX, makeindex) CMD_DEFINE(TEXINDY, texindy) CMD_DEFINE(METAPOST, metapost) CMD_DEFINE(ASY, asy) CMD_DEFINE(BIBTEX, bibtex) CMD_DEFINE(SVN, svn) CMD_DEFINE(SVNADMIN, svnadmin)
-CMD_DEFINE(COMPILE, compile) CMD_DEFINE(VIEW, view) CMD_DEFINE(BIBLIOGRAPHY, bibliography) CMD_DEFINE(INDEX, index) CMD_DEFINE(QUICK, quick) CMD_DEFINE(RECOMPILE_BIBLIOGRAPHY, recompile-bibliography)
+CMD_DEFINE(DVIPNG, dvipng) CMD_DEFINE(DVIPS, dvips) CMD_DEFINE(DVIPDF, dvipdf) CMD_DEFINE(PS2PDF, ps2pdf) CMD_DEFINE(GS, gs) CMD_DEFINE(MAKEINDEX, makeindex) CMD_DEFINE(TEXINDY, texindy) CMD_DEFINE(MAKEGLOSSARIES, makeglossaries) CMD_DEFINE(METAPOST, metapost) CMD_DEFINE(ASY, asy) CMD_DEFINE(BIBTEX, bibtex) CMD_DEFINE(BIBTEX8, bibtex8) CMD_DEFINE(BIBER, biber) CMD_DEFINE(SVN, svn) CMD_DEFINE(SVNADMIN, svnadmin)
+CMD_DEFINE(COMPILE, compile) CMD_DEFINE(VIEW, view) CMD_DEFINE(BIBLIOGRAPHY, bibliography) CMD_DEFINE(INDEX, index) CMD_DEFINE(GLOSSARY, glossary) CMD_DEFINE(QUICK, quick) CMD_DEFINE(RECOMPILE_BIBLIOGRAPHY, recompile-bibliography)
 CMD_DEFINE(VIEW_PDF_INTERNAL, view-pdf-internal) CMD_DEFINE(CONDITIONALLY_RECOMPILE_BIBLIOGRAPHY, conditionally-recompile-bibliography)
 CMD_DEFINE(INTERNAL_PRE_COMPILE, internal-pre-compile)
 #undef CMD_DEFINE
@@ -131,18 +131,22 @@ QString BuildManager::chainCommands(const QString& a, const QString& b, const QS
 
 
 BuildManager::BuildManager(): processWaitedFor(0)
-     #ifdef Q_WS_WIN
+     #ifdef Q_OS_WIN32
      , pidInst(0)
      #endif
 {
 	initDefaultCommandNames();
 	connect(this, SIGNAL(commandLineRequested(QString,QString*,bool*)), SLOT(commandLineRequestedDefault(QString,QString*,bool*)));
+
+	m_stopBuildAction = new QAction(getRealIcon("stop"), tr("Stop Compile"), this);
+	connect(m_stopBuildAction, SIGNAL(triggered()), this, SLOT(killCurrentProcess()));
+	m_stopBuildAction->setEnabled(false);
 }
 BuildManager::~BuildManager() {
 	//remove preview file names
 	foreach(const QString& elem,previewFileNames)
 		removePreviewFiles(elem);
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	if (pidInst) DdeUninitialize(pidInst);
 #endif
 }
@@ -167,7 +171,7 @@ void BuildManager::initDefaultCommandNames(){
 	registerCommand("biber",       "biber",        "Biber" ,       "%"); //todo: correct parameter?
 	registerCommand("makeindex",   "makeindex",    "Makeindex",   "%.idx", "Tools/Makeindex");
 	registerCommand("texindy",     "texindy",      "Texindy", "%.idx");
-	registerCommand("makeglossary","makeglossary;makeglossaries", "Makeglossary", "%");
+	registerCommand("makeglossaries","makeglossaries", "Makeglossaries", "%");
 	registerCommand("metapost",    "mpost",        "Metapost",    "-interaction=nonstopmode ?me)", "Tools/Metapost");
 	registerCommand("asy",         "asy",          "Asymptote",   "?m*.asy", "Tools/Asy");
 	registerCommand("gs",          "gs;mgs",           "Ghostscript", "\"?am.ps\"", "Tools/Ghostscript", &getCommandLineGhostscript);
@@ -191,10 +195,13 @@ void BuildManager::initDefaultCommandNames(){
 	registerCommand("view-pdf", tr("PDF Viewer"), QStringList() << "txs:///view-pdf-internal --embedded" << "txs:///view-pdf-internal" << "txs:///view-pdf-external","",true,descriptionList);
 	descriptionList.clear();
 	descriptionList<< tr("BibTeX") << tr("BibTeX 8-Bit") << tr("Biber");
-	registerCommand("bibliography", tr("Default Bibliography"), QStringList() << "txs:///bibtex" << "txs:///bibtex8" << "txs:///biber","",true,descriptionList);
+	registerCommand("bibliography", tr("Default Bibliography Tool"), QStringList() << "txs:///bibtex" << "txs:///bibtex8" << "txs:///biber","",true,descriptionList);
 	descriptionList.clear();
 	descriptionList<< tr("BibTeX") << tr("BibTeX 8-Bit") << tr("Biber");
 	registerCommand("index", tr("Default Index Tool"), QStringList() << "txs:///makeindex" << "txs:///texindy","",true,descriptionList);
+	descriptionList.clear();
+	descriptionList<< tr("Makeglossaries");
+	registerCommand("glossary", tr("Default Glossary Tool"), QStringList() << "txs:///makeglossaries","",true,descriptionList);
 	
 	registerCommand("ps-chain", tr("PS Chain"), QStringList() << "txs:///latex | txs:///dvips | txs:///view-ps");
 	registerCommand("dvi-chain", tr("DVI Chain"), QStringList() << "txs:///latex | txs:///view-dvi");
@@ -356,7 +363,7 @@ QStringList BuildManager::parseExtendedCommandLine(QString str, const QFileInfo
 
 
 QString BuildManager::findFileInPath(QString fileName) {
-/*#ifdef Q_WS_MAC
+/*#ifdef Q_OS_MAC
 	QProcess *myProcess = new QProcess();
 	myProcess->start("bash -l -c \"echo $PATH\"");
 	myProcess->waitForFinished(3000);
@@ -370,7 +377,7 @@ QString BuildManager::findFileInPath(QString fileName) {
 	if (i==-1) return "";
 	QString path=env[i].mid(5); //skip path=
 //#endif
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	if (!fileName.contains('.')) fileName+=".exe";
 	QStringList paths=path.split(";"); //windows
 #else
@@ -383,8 +390,8 @@ QString BuildManager::findFileInPath(QString fileName) {
 	return "";
 }
 
-#ifdef Q_WS_WIN
-typedef BOOL (* AssocQueryStringAFunc)(DWORD, DWORD, const char*, const char*, char*, DWORD*);
+#ifdef Q_OS_WIN32
+typedef BOOL (__stdcall * AssocQueryStringAFunc)(DWORD, DWORD, const char*, const char*, char*, DWORD*);
 QString W32_FileAssociation(QString ext) {
 	if (ext=="") return "";
 	if (ext[0]!=QChar('.')) ext='.'+ext;
@@ -398,7 +405,7 @@ QString W32_FileAssociation(QString ext) {
 		DWORD buflen=1023;
 		if (assoc(0, ASSOCSTR_COMMAND, ba.data(), "open", &buf[0], &buflen)==S_OK) {
 			buf[buflen]=0;
-			result=QString::fromAscii(buf);
+			result=QString::fromLatin1(buf);
 			result.replace("%1","?am"+ext);
 			//QMessageBox::information(0,result,result,0);
 		};
@@ -445,15 +452,14 @@ QString getTeXLiveBinPath() {
 	foreach (const QString& baseKey, QStringList() << "HKEY_CURRENT_USER" << "HKEY_LOCAL_MACHINE") {
 		QSettings reg(baseKey+ "\\Software", QSettings::NativeFormat);
 		QString uninstall;
-		for (int v=2013; v>2008; v--) {
-			uninstall = reg.value("microsoft/windows/currentversion/uninstall/TeXLive2010/UninstallString", "").toString();
-			if (!uninstall.isEmpty()) break;
-		}
-		if (!uninstall.isEmpty()) {
-			int p = uninstall.indexOf("\\tlpkg\\", 0, Qt::CaseInsensitive);
-			QString path = p>0?uninstall.left(p):"";
-			if (QDir(path+"\\bin\\win32").exists())
-				return path+"\\bin\\win32\\";
+		for (int v=2014; v>2008; v--) {
+			uninstall = reg.value(QString("microsoft/windows/currentversion/uninstall/TeXLive%1/UninstallString").arg(v), "").toString();
+			if (!uninstall.isEmpty()) {
+				int p = uninstall.indexOf("\\tlpkg\\", 0, Qt::CaseInsensitive);
+				QString path = p > 0 ? uninstall.left(p) : "";
+				if (QDir(path+"\\bin\\win32").exists())
+					return path+"\\bin\\win32\\";
+			}
 		}
 	}
 	//check for path
@@ -502,7 +508,7 @@ QString searchBaseCommand(const QString &cmd, QString options) {
 			return fileName+options; //found in path
 		else {
 			//platform dependent mess
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 			//Windows MikTex
 			QString mikPath=getMiKTeXBinPath();
 			if (!mikPath.isEmpty() && QFileInfo(mikPath+fileName).exists())
@@ -512,7 +518,7 @@ QString searchBaseCommand(const QString &cmd, QString options) {
 			if (!livePath.isEmpty() && QFileInfo(livePath+fileName).exists())
 				return "\""+livePath+fileName+"\" "+options; //found
 #endif
-#ifdef Q_WS_MACX
+#ifdef Q_OS_MAC
 			QStringList paths;
 			paths << "/usr/bin/texbin/" << "/usr/local/bin/" << "/usr/texbin/";
 			paths << "/usr/local/teTeX/bin/i386-apple-darwin-current/" << "/usr/local/teTeX/bin/powerpc-apple-darwin-current/" << "/usr/local/teTeX/bin/x86_64-apple-darwin-current/";
@@ -524,8 +530,14 @@ QString searchBaseCommand(const QString &cmd, QString options) {
                 paths << QString("/usr/local/texlive/%1/bin/powerpc-darwin/").arg(i);
 			}
 			foreach (const QString& p, paths)
-				if (QFileInfo(p+fileName).exists())
-					return p+fileName+options;
+				if (QFileInfo(p+fileName).exists()) {
+					if (cmd=="makeglossaries") {
+						// workaround: makeglossaries calls makeindex or xindy and therefore has to be run in an environment that has these commands on the path
+						return QString("sh -c \"PATH=$PATH:%1; %2%3\"").arg(p).arg(fileName).arg(options);
+					} else {
+						return p+fileName+options;
+					}
+				}
 #endif
 		}
 	}
@@ -714,14 +726,14 @@ bool BuildManager::hasCommandLine(const QString& program){
 	return false;
 }
 
-#if defined(Q_WS_MACX)
+#if defined(Q_OS_MAC)
 
 QString getCommandLineViewDvi(){ return "open %.dvi > /dev/null"; }
 QString getCommandLineViewPs(){ return "open %.ps > /dev/null"; }
 QString getCommandLineViewPdfExternal(){ return "open %.pdf > /dev/null"; }
 QString getCommandLineGhostscript(){ return ""; }
 
-#elif defined(Q_WS_WIN)
+#elif defined(Q_OS_WIN32)
 
 QString getCommandLineViewDvi(){ 
 	const QString yapOptions = " -1 -s @?\"c:ame \"?am.dvi\"";
@@ -999,7 +1011,7 @@ void BuildManager::saveSettings(QSettings &settings){
 	settings.endGroup();
 }
 
-bool BuildManager::runCommand(const QString &unparsedCommandLine, const QFileInfo &mainFile, const QFileInfo &currentFile, int currentLine, QString* buffer){
+bool BuildManager::runCommand(const QString &unparsedCommandLine, const QFileInfo &mainFile, const QFileInfo &currentFile, int currentLine, QString* buffer, QTextCodec* codecForBuffer ){
 	if (waitingForProcess()) return false;
 	if (unparsedCommandLine.isEmpty()) { emit processNotification(tr("Error: No command given")); return false; }
 	ExpandingOptions options(mainFile, currentFile, currentLine);
@@ -1022,12 +1034,12 @@ bool BuildManager::runCommand(const QString &unparsedCommandLine, const QFileInf
 	bool asyncPdf = !(expansion.commands.last().flags & RCF_WAITFORFINISHED) && (expansion.commands.last().flags & RCF_CHANGE_PDF);
 	
 	emit beginRunningCommands(expansion.primaryCommand, latexCompiled, pdfChanged, asyncPdf);
-	bool result = runCommandInternal(expansion, mainFile, buffer);
+	bool result = runCommandInternal(expansion, mainFile, buffer, codecForBuffer);
 	emit endRunningCommands(expansion.primaryCommand, latexCompiled, pdfChanged, asyncPdf);
 	return result;
 }
 
-bool BuildManager::runCommandInternal(const ExpandedCommands& expandedCommands, const QFileInfo &mainFile, QString* buffer){
+bool BuildManager::runCommandInternal(const ExpandedCommands& expandedCommands, const QFileInfo &mainFile, QString* buffer, QTextCodec* codecForBuffer){
 	const QList<CommandToRun> & commands = expandedCommands.commands;
 	
 	int remainingReRunCount = autoRerunLatex;
@@ -1051,6 +1063,7 @@ bool BuildManager::runCommandInternal(const ExpandedCommands& expandedCommands,
 
 		
 		p->setStdoutBuffer(buffer);
+		p->setStdoutCodec(codecForBuffer);
 		
 		emit beginRunningSubCommand(p, expandedCommands.primaryCommand, cur.parentCommand, cur.flags);
 		
@@ -1077,7 +1090,11 @@ bool BuildManager::runCommandInternal(const ExpandedCommands& expandedCommands,
 			if (result == LCR_ERROR) return false;
 			if (result == LCR_NORMAL || !rerunnable) continue;
 			if (remainingReRunCount <= 0) continue; //do not abort since the rerun condition might have been trigged accidentally
-			if (result == LCR_RERUN_WITH_BIBLIOGRAPHY) { runCommand(CMD_BIBLIOGRAPHY, mainFile, mainFile, 0); remainingReRunCount--;}
+			if (result == LCR_RERUN_WITH_BIBLIOGRAPHY) {
+				QString tempWaitForFinished; //if it does not wait on bibtex it will fail
+				runCommand(CMD_BIBLIOGRAPHY, mainFile, mainFile, 0, &tempWaitForFinished);
+				remainingReRunCount--;
+			}
 			REQUIRE_RET(result == LCR_RERUN || result == LCR_RERUN_WITH_BIBLIOGRAPHY, false);
 			remainingReRunCount--;
 			i--; //rerun
@@ -1119,10 +1136,10 @@ ProcessX* BuildManager::newProcessInternal(const QString &cmd, const QFileInfo&
 	if(!mainFile.fileName().isEmpty())
 		proc->setWorkingDirectory(mainFile.absolutePath());
 	if (cmd.startsWith(TXS_CMD_PREFIX)) 
-		connect(proc, SIGNAL(started()), SLOT(runInternalCommandThroughProcessX()));
+		connect(proc, SIGNAL(startedX()), SLOT(runInternalCommandThroughProcessX()));
 	
 	QString addPaths = additionalSearchPaths;
-#ifdef Q_WS_MACX
+#ifdef Q_OS_MAC
 #if (QT_VERSION >= 0x040600)
 	QProcess *myProcess = new QProcess();
 	myProcess->start("bash -l -c \"echo $PATH\"");
@@ -1140,7 +1157,7 @@ ProcessX* BuildManager::newProcessInternal(const QString &cmd, const QFileInfo&
 
 	if (!addPaths.isEmpty()) {
 		QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-		env.insert("PATH", env.value("PATH") + ":"+addPaths); //apply user path as well
+		env.insert("PATH", env.value("PATH") + getPathListSeparator() + addPaths); //apply user path as well
 		proc->setProcessEnvironment(env);		
 	}	
 	
@@ -1150,6 +1167,7 @@ ProcessX* BuildManager::newProcessInternal(const QString &cmd, const QFileInfo&
 bool BuildManager::waitForProcess(ProcessX* p){
 	REQUIRE_RET(!processWaitedFor, false);
 	processWaitedFor = p;
+	m_stopBuildAction->setEnabled(true);
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 	QTime time;
 	time.start();
@@ -1165,6 +1183,7 @@ bool BuildManager::waitForProcess(ProcessX* p){
 	QApplication::restoreOverrideCursor();
 	bool result = processWaitedFor;
 	processWaitedFor = 0;
+	m_stopBuildAction->setEnabled(false);
 	return result;
 }
 
@@ -1268,7 +1287,7 @@ void BuildManager::preview(const QString &preamble, const PreviewSource& source,
 				connect(p,SIGNAL(finished(int)),this,SLOT(preamblePrecompileCompleted(int)));
 				connect(p,SIGNAL(finished(int)),p,SLOT(deleteLater()));
 				tf->setParent(p); //free file when process is deleted
-				REQUIRE(p);
+
 				p->startCommand();
 				
 				if (p->waitForStarted()) {
@@ -1310,7 +1329,7 @@ void BuildManager::preview(const QString &preamble, const PreviewSource& source,
 	// start conversion
 	// tex -> dvi
 	ProcessX *p1 = firstProcessOfDirectExpansion(CMD_LATEX, QFileInfo(ffn)); //no delete! goes automatically
-	REQUIRE(p1);
+    if(!p1) return; // command failed, not set ?
 	addLaTeXInputPaths(p1, addPaths);
 	connect(p1,SIGNAL(finished(int)),this,SLOT(latexPreviewCompleted(int)));
 	p1->startCommand();
@@ -1320,7 +1339,7 @@ void BuildManager::preview(const QString &preamble, const PreviewSource& source,
 		// dvi -> png
 		//follow mode is a tricky features which allows dvipng to run while tex isn't finished
 		ProcessX *p2 = firstProcessOfDirectExpansion("txs:///dvipng/[--follow]", ffn);
-		REQUIRE(p2);
+        if(!p2) return; // command failed, not set ?
 		if (!p1->overrideEnvironment().isEmpty()) p2->setOverrideEnvironment(p1->overrideEnvironment());
 		connect(p2,SIGNAL(finished(int)),this,SLOT(conversionPreviewCompleted(int)));
 		p2->startCommand();
@@ -1435,7 +1454,8 @@ void BuildManager::latexPreviewCompleted(int status){
 		if (!p1) return;
 		// dvi -> png
 		ProcessX *p2 = firstProcessOfDirectExpansion(CMD_DVIPNG,p1->getFile());
-		REQUIRE(p2);
+        if(!p2) return; //dvipng does not work
+        //REQUIRE(p2);
 		if (!p1->overrideEnvironment().isEmpty()) p2->setOverrideEnvironment(p1->overrideEnvironment());
 		connect(p2,SIGNAL(finished(int)),this,SLOT(conversionPreviewCompleted(int)));
 		p2->startCommand();
@@ -1445,7 +1465,8 @@ void BuildManager::latexPreviewCompleted(int status){
 		if (!p1) return;
 		// dvi -> ps
 		ProcessX *p2 = firstProcessOfDirectExpansion("txs:///dvips/[-E]", p1->getFile());
-		REQUIRE(p2);
+        if(!p2) return; //dvips does not work
+        //REQUIRE(p2);
 		if (!p1->overrideEnvironment().isEmpty()) p2->setOverrideEnvironment(p1->overrideEnvironment());
 		connect(p2,SIGNAL(finished(int)),this,SLOT(dvi2psPreviewCompleted(int)));
 		p2->startCommand();
@@ -1521,7 +1542,7 @@ QString BuildManager::findFile(const QString& defaultName, const QString& search
 	QString baseName = "/" + base.fileName();
 	              
 	
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	QString dirSep = ";";
 #else
 	QString dirSep = ":";
@@ -1548,7 +1569,7 @@ void BuildManager::removePreviewFiles(QString elem){
 
 
 //DDE things
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 #include "windows.h"
 bool BuildManager::executeDDE(QString ddePseudoURL) {
 	//parse URL
@@ -1640,7 +1661,7 @@ bool BuildManager::executeDDE(QString ddePseudoURL) {
 #endif
 
 ProcessX::ProcessX(BuildManager* parent, const QString &assignedCommand, const QString& fileToCompile):
-    QProcess(parent), cmd(assignedCommand.trimmed()), file(fileToCompile), isStarted(false), ended(false), stderrEnabled(true), stdoutEnabled(true), stdoutEnabledOverrideOn(false), stdoutBuffer(0) {
+	QProcess(parent), cmd(assignedCommand.trimmed()), file(fileToCompile), isStarted(false), ended(false), stderrEnabled(true), stdoutEnabled(true), stdoutEnabledOverrideOn(false), stdoutBuffer(0), stdoutCodec(0) {
 	QString stdoutRedirection = cmd.mid(cmd.lastIndexOf(">") + 1).trimmed();
 	if (stdoutRedirection == "/dev/null" || stdoutRedirection == "txs:///messages"  )  {
 		cmd = cmd.left(cmd.lastIndexOf(">")).trimmed();
@@ -1654,7 +1675,7 @@ ProcessX::ProcessX(BuildManager* parent, const QString &assignedCommand, const Q
 void ProcessX::startCommand() {
 	ended = false;
 	
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	if (cmd.startsWith("dde://") || cmd.startsWith("dde:///")) {
 		onStarted();
 		BuildManager* manager = qobject_cast<BuildManager*>(parent());
@@ -1671,7 +1692,8 @@ void ProcessX::startCommand() {
 #endif
 	
 	if (cmd.startsWith("txs:///")){
-        emit started();
+		onStarted();
+		emit startedX();
 		emit finished(0);
 		emit finished(0, NormalExit);
 		return;
@@ -1683,8 +1705,11 @@ void ProcessX::startCommand() {
 	
     //qDebug() << workingDirectory();
     //qDebug() << cmd;
+	QByteArray path = qgetenv("PATH");
+    qputenv("PATH", path + getPathListSeparator().toLatin1() + BuildManager::additionalSearchPaths.toUtf8()); // needed for searching the executable in the additional paths see https://bugreports.qt-project.org/browse/QTBUG-18387
 	QProcess::start(cmd);
-	
+	qputenv("PATH", path); // restore
+
 #ifdef PROFILE_PROCESSES
 	connect(this, SIGNAL(finished(int)), SLOT(finished()));
 	time.start();
@@ -1713,6 +1738,9 @@ QString * ProcessX::getStdoutBuffer(){
 void ProcessX::setStdoutBuffer(QString *buffer){
 	stdoutBuffer = buffer;
 }
+void ProcessX::setStdoutCodec(QTextCodec *codec){
+	stdoutCodec = codec;
+}
 bool ProcessX::showStderr() const{
 	return stderrEnabled;
 }
@@ -1732,8 +1760,8 @@ const QStringList& ProcessX::overrideEnvironment(){
 
 int ProcessX::exitStatus() const{return QProcess::exitStatus();}
 int ProcessX::exitCode() const{return QProcess::exitCode(); }
-QString ProcessX::readAllStandardOutputStr(){return QString::fromLocal8Bit(QProcess::readAllStandardOutput());}
-QString ProcessX::readAllStandardErrorStr(){return QString::fromLocal8Bit(QProcess::readAllStandardError());}
+QString ProcessX::readAllStandardOutputStr(){return stdoutCodec ? stdoutCodec->toUnicode(QProcess::readAllStandardOutput()) : QString::fromLocal8Bit(QProcess::readAllStandardOutput());}
+QString ProcessX::readAllStandardErrorStr(){return stdoutCodec ? stdoutCodec->toUnicode(QProcess::readAllStandardError()) : QString::fromLocal8Bit(QProcess::readAllStandardError());}
 bool ProcessX::waitForFinished( int msecs ){return QProcess::waitForFinished(msecs);}
 
 bool ProcessX::isRunning() const{
@@ -1743,6 +1771,7 @@ bool ProcessX::isRunning() const{
 void ProcessX::onStarted(){
 	if (isStarted) return; //why am I called twice?
 	isStarted=true;
+	emit startedX();
 	emit processNotification(tr("Process started: %1").arg(cmd));
 }
 
