@@ -108,8 +108,8 @@ public:
 	static const QString TXS_CMD_PREFIX;
 	static const QString CMD_LATEX, CMD_PDFLATEX, CMD_XELATEX, CMD_LUALATEX,CMD_LATEXMK;
 	static const QString CMD_VIEW_DVI, CMD_VIEW_PS, CMD_VIEW_PDF, CMD_VIEW_LOG;
-	static const QString CMD_DVIPNG, CMD_DVIPS, CMD_DVIPDF, CMD_PS2PDF, CMD_GS, CMD_MAKEINDEX, CMD_TEXINDY, CMD_METAPOST, CMD_ASY, CMD_BIBTEX, CMD_SVN, CMD_SVNADMIN;
-	static const QString CMD_COMPILE, CMD_VIEW, CMD_BIBLIOGRAPHY, CMD_INDEX, CMD_QUICK, CMD_RECOMPILE_BIBLIOGRAPHY;
+	static const QString CMD_DVIPNG, CMD_DVIPS, CMD_DVIPDF, CMD_PS2PDF, CMD_GS, CMD_MAKEINDEX, CMD_TEXINDY, CMD_MAKEGLOSSARIES, CMD_METAPOST, CMD_ASY, CMD_BIBTEX, CMD_BIBTEX8, CMD_BIBER, CMD_SVN, CMD_SVNADMIN;
+	static const QString CMD_COMPILE, CMD_VIEW, CMD_BIBLIOGRAPHY, CMD_INDEX, CMD_GLOSSARY, CMD_QUICK, CMD_RECOMPILE_BIBLIOGRAPHY;
 	static const QString CMD_VIEW_PDF_INTERNAL, CMD_INTERNAL_PRE_COMPILE, CMD_CONDITIONALLY_RECOMPILE_BIBLIOGRAPHY;
 	
 	static QString chainCommands(const QString& a);
@@ -126,11 +126,14 @@ public:
 	void registerOptions(ConfigManagerInterface& cmi);
 	void readSettings(QSettings &settings);
 	void saveSettings(QSettings &settings);
+
+	QAction * stopBuildAction() {return m_stopBuildAction;}
 	
 public slots:
-	bool runCommand(const QString &unparsedCommandLine, const QFileInfo &mainFile, const QFileInfo &currentFile = QFileInfo(), int currentLine = 0, QString* buffer = 0);
+	bool runCommand(const QString &unparsedCommandLine, const QFileInfo &mainFile, const QFileInfo &currentFile = QFileInfo(), int currentLine = 0, QString* buffer = 0, QTextCodec *codecForBuffer = 0);
+	Q_INVOKABLE void killCurrentProcess();
 private:
-	bool runCommandInternal(const ExpandedCommands& expandedCommands, const QFileInfo &mainFile, QString* buffer = 0);
+	bool runCommandInternal(const ExpandedCommands& expandedCommands, const QFileInfo &mainFile, QString* buffer = 0, QTextCodec *codecForBuffer = 0);
 public:
 	//creates a process object with the given command line (after it is changed by an implcit call to parseExtendedCommandLine)
 	//ProcessX* newProcess(const QString &unparsedCommandLine, const QString &mainFile, const QString &currentFile, int currentLine=0, bool singleInstance = false);
@@ -141,7 +144,6 @@ public:
 public:
 	Q_INVOKABLE bool waitForProcess(ProcessX* p);
 	Q_INVOKABLE bool waitingForProcess() const;
-	Q_INVOKABLE void killCurrentProcess(); 
 	
 	static QString createTemporaryFileName(); //don't forget to remove the file!
 	
@@ -213,11 +215,12 @@ public:
     }
 
 private:
+	QAction * m_stopBuildAction;
 	QStringList previewFileNames;
 	QMap<QString, PreviewSource> previewFileNameToSource;
 	QHash<QString, QString> preambleHash;
 	void removePreviewFiles(QString elemName);
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	unsigned long int pidInst;
 	Q_INVOKABLE bool executeDDE(QString ddePseudoURL);
 #endif
@@ -238,6 +241,7 @@ public:
 	void setShowStdout(bool show);
 	QString * getStdoutBuffer();
 	void setStdoutBuffer(QString *buffer);
+	void setStdoutCodec(QTextCodec *codec);
 	bool showStderr() const;
 	void setShowStderr(bool show);
 	void setOverrideEnvironment(const QStringList& env);
@@ -251,9 +255,8 @@ public:
 	
 	bool isRunning() const;
 signals:
-#if (QT_VERSION >= 0x050000)
-    void started();
-#endif
+	void startedX();
+
 	void processNotification(const QString& message);
 	void standardOutputRead(const QString& data);
 	void standardErrorRead(const QString& data);
@@ -272,6 +275,7 @@ private:
 	QString file;
 	bool isStarted, ended, stderrEnabled, stdoutEnabled, stdoutEnabledOverrideOn;
 	QString *stdoutBuffer;
+	QTextCodec *stdoutCodec;
 	QStringList overriddenEnvironment;
 	QString subCommandName, subCommandPrimary;
 	RunCommandFlags subCommandFlags;
