@@ -10,6 +10,7 @@
  ***************************************************************************/
 
 #include "configdialog.h"
+#include "configmanager.h"
 
 #include "updatechecker.h"
 
@@ -56,7 +57,15 @@ ShortcutComboBox::ShortcutComboBox(QWidget *parent):QComboBox(parent){
 
 void ShortcutComboBox::keyPressEvent(QKeyEvent *e){
 	if ( (e->modifiers()!=0 && e->text() != "+" && e->key() != Qt::Key_Alt && e->key() != Qt::Key_Shift && e->key() != Qt::Key_Control && e->key() != Qt::Key_AltGr && e->key() != Qt::Key_Meta && e->key() != 0 && e->key() != Qt::Key_Super_L && e->key() != Qt::Key_Super_R)
-		|| (e->key() >= Qt::Key_F1 &&  e->key() <= Qt::Key_F35)) {
+        || (e->key() >= Qt::Key_F1 &&  e->key() <= Qt::Key_F35)
+        || (e->key() == Qt::Key_Home)
+        || (e->key() == Qt::Key_End)
+        || (e->key() == Qt::Key_PageUp)
+        || (e->key() == Qt::Key_PageDown)
+        || (e->key() == Qt::Key_Up)
+        || (e->key() == Qt::Key_Down)
+         )
+    {
 		// FIXME: Qt currently does not handle KeypadModifier correctly.
 		// as a workaround we just take it away, so there is no difference between keypad and non-keypad keys,
 		// but at least keypad keys don't produce rubbish. See also sf.net bug item 3525266
@@ -172,8 +181,9 @@ void ShortcutDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 			txsWarning(ConfigDialog::tr("The shortcut you entered is invalid."));
 			return;
 		}
+        QString value_alternative=QKeySequence(box->currentText()).toString(QKeySequence::PortableText);
 		QRegExp rxCharKey("(Shift\\+)?."); // matches all single characters and single characters with shift like "Shift+A", should not match e.g. "F1" or "DEL"
-		if (rxCharKey.exactMatch(value)) {
+        if (rxCharKey.exactMatch(value_alternative)) {
 			if (!txsConfirmWarning(ConfigDialog::tr("The shortcut you entered is a standard character key.\n"
 													"You will not be able to type this character. Do you wish\n"
 													"to set the key anyway?"))) {
@@ -331,6 +341,7 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 	//ui.contentsWidget->setMovement(QListView::Static);
 
 	//pageGeneral
+	connect(ui.pushButtonImportDictionary, SIGNAL(clicked()), this, SLOT(importDictionary()));
 	connect(ui.pushButtonUpdateCheckNow, SIGNAL(clicked()), this, SLOT(updateCheckNow()));
 	connect(UpdateChecker::instance(), SIGNAL(checkCompleted()), this, SLOT(refreshLastUpdateTime()));
 	refreshLastUpdateTime();
@@ -350,11 +361,13 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 	ui.comboBoxThesaurusFileName->setCompleter(0);
 
 	connect(ui.pushButtonDictDir, SIGNAL(clicked()), this, SLOT(browseDictDir()));
-	connect(ui.leDictDir, SIGNAL(textChanged(QString)), this, SLOT(dictDirChanged(QString)));
+	connect(ui.leDictDir, SIGNAL(textChanged(QString)), this, SLOT(updateDefaultDictSelection(QString)));
 
 	connect(ui.btSelectThesaurusFileName, SIGNAL(clicked()), this, SLOT(browseThesaurus()));
 
 	connect(ui.pushButtonPathLog, SIGNAL(clicked()), this, SLOT(browsePathLog()));
+	connect(ui.pushButtonPathBib, SIGNAL(clicked()), this, SLOT(browsePathBib()));
+	connect(ui.pushButtonPathImages, SIGNAL(clicked()), this, SLOT(browsePathImages()));
 	connect(ui.pushButtonPathPdf, SIGNAL(clicked()), this, SLOT(browsePathPdf()));
 	connect(ui.pushButtonPathCommands, SIGNAL(clicked()), this, SLOT(browsePathCommands()));
 	
@@ -362,7 +375,7 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 	connect(ui.comboBoxThesaurusFileName, SIGNAL(highlighted(QString)), this, SLOT(comboBoxWithPathHighlighted(QString)));
 
 
-	ui.labelGetDic->setText(tr("Download Additional Dictionaries: %1").arg("<br><a href=\"http://wiki.services.openoffice.org/wiki/Dictionaries\">http://wiki.services.openoffice.org/wiki/Dictionaries</a>"));
+	ui.labelGetDic->setText(tr("Download additional dictionaries from %1 or %2").arg("<a href=\"http://extensions.openoffice.org/de/search?f[0]=field_project_tags%3A157\">OpenOffice</a>").arg("<a href=\"http://extensions.libreoffice.org/extension-center?getCategories=Dictionary\">LibreOffice</a>"));
 	ui.labelGetDic->setOpenExternalLinks(true);
 	//page custom environment
 	connect(ui.pbAddLine, SIGNAL(clicked()), this, SLOT(custEnvAddLine()));
@@ -378,12 +391,12 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 
 	fmConfig=new QFormatConfig(ui.formatConfigBox);
 	fmConfig->addCategory(tr("Basic highlighting")) << "normal" <<"background" <<"comment" <<"commentTodo" <<"keyword" <<"extra-keyword" <<"math-delimiter" <<"math-keyword" <<"numbers" <<"text" <<"align-ampersand" <<"environment" <<"structure" <<"link" <<"escapeseq" << "verbatim" << "picture" << "preedit";
-	fmConfig->addCategory(tr("LaTeX checking")) <<"braceMatch" <<"braceMismatch" <<"latexSyntaxMistake" <<"referencePresent" <<"referenceMissing" <<"referenceMultiple" <<"citationPresent" <<"citationMissing"  <<"temporaryCodeCompletion";
+    fmConfig->addCategory(tr("LaTeX checking")) <<"braceMatch" <<"braceMismatch" <<"latexSyntaxMistake" <<"referencePresent" <<"referenceMissing" <<"referenceMultiple" <<"citationPresent" <<"citationMissing" <<"packagePresent" <<"packageMissing" <<"temporaryCodeCompletion";
 	fmConfig->addCategory(tr("Language checking")) <<"spellingMistake" << "wordRepetition" << "wordRepetitionLongRange"<< "badWord" << "grammarMistake" << "grammarMistakeSpecial1" << "grammarMistakeSpecial2" << "grammarMistakeSpecial3" << "grammarMistakeSpecial4";
 	fmConfig->addCategory(tr("Line highlighting"))     <<"line:error" <<"line:warning" <<"line:badbox" <<"line:bookmark" <<"line:bookmark0" <<"line:bookmark1" <<"line:bookmark2" <<"line:bookmark3" <<"line:bookmark4" <<"line:bookmark5" <<"line:bookmark6"  <<"line:bookmark7" <<"line:bookmark8"<<"line:bookmark9"<<"current";
 	fmConfig->addCategory(tr("Search")) <<"search"<<"replacement"<<"selection";
 	fmConfig->addCategory(tr("Diff")) <<"diffDelete"<<"diffAdd"<<"diffReplace";
-	fmConfig->addCategory(tr("DTX files")) << "dtx:guard" << "dtx:macro" << "dtx:verbatim" << "dtx:specialchar";
+	fmConfig->addCategory(tr("DTX files")) << "dtx:guard" << "dtx:macro" << "dtx:verbatim" << "dtx:specialchar" << "dtx:commands";
 	fmConfig->addCategory(tr("Sweave / Pweave")) << "sweave-delimiter" << "sweave-block" << "pweave-delimiter" << "pweave-block";
 	fmConfig->addCategory(tr("Lua")) << "lua:keyword" << "lua:comment";
 
@@ -447,8 +460,6 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 	ui.checkBoxUseSystemTheme->setVisible(false);
 #endif
 	
-	ui.scrollAreaPaths->setVisible(false);
-	
 	QRect screen = QApplication::desktop()->screenGeometry();
 	if (!screen.isEmpty()) {
 		int nwidth = width(), nheight = height();
@@ -562,6 +573,15 @@ void ConfigDialog::browseDictDir() {
 void ConfigDialog::browsePathLog(){
 	browse(ui.lineEditPathLog, tr("Search Path for Logs"), "/", QDir::currentPath(), true);
 }
+
+void ConfigDialog::browsePathBib(){
+	browse(ui.lineEditPathBib, tr("Search Path .bib Files"), "/", QDir::currentPath(), true);
+}
+
+void ConfigDialog::browsePathImages(){
+	browse(ui.lineEditPathImages, tr("Search Path for Images"), "/", QDir::currentPath(), true);
+}
+
 void ConfigDialog::browsePathPdf(){
 	browse(ui.lineEditPathPDF, tr("Search Path for PDFs"), "/", QDir::currentPath(), true);
 }
@@ -569,12 +589,23 @@ void ConfigDialog::browsePathCommands(){
 	browse(ui.lineEditPathCommands, tr("Search Path for Commands"), "/", QDir::rootPath(), true);
 }
 
-void ConfigDialog::dictDirChanged(const QString &newText) {
+void ConfigDialog::updateDefaultDictSelection(const QString &dictPaths, const QString &newDefault) {
 	QString lang = ui.comboBoxSpellcheckLang->currentText();
 	ui.comboBoxSpellcheckLang->clear();
-	ui.comboBoxSpellcheckLang->addItems(SpellerManager::dictNamesForDir(newText));
-	// keep selected language if possible
-	int index = ui.comboBoxSpellcheckLang->findText(lang);
+
+	ConfigManager *config = dynamic_cast<ConfigManager *>(ConfigManagerInterface::getInstance());
+	if (!config) return;
+	SpellerManager sm;
+	sm.setDictPaths(config->parseDirList(dictPaths));
+	ui.comboBoxSpellcheckLang->addItems(sm.availableDicts());
+	if (ui.comboBoxSpellcheckLang->count() == 0) {
+		ui.comboBoxSpellcheckLang->addItem("<none>");
+	}
+
+	// update selected language
+	int index = -1;
+	if (!newDefault.isNull()) index = ui.comboBoxSpellcheckLang->findText(newDefault);
+	if (index < 0) index = ui.comboBoxSpellcheckLang->findText(lang); // keep selected language
 	if (index >=0) {
 		ui.comboBoxSpellcheckLang->setCurrentIndex(index);
 	}
@@ -734,7 +765,7 @@ void ConfigDialog::loadOtherIcon(){
 		txsWarning(tr("You need to add an action to the toolbar (from the list of actions on the right side), before you can load an icon for that item."));
 		return;
 	}
-	QString fn = QFileDialog::getOpenFileName(this,tr("Select a File"),"",tr("Images (*.png *.xpm *.jpg *.bmp *.svg)"));
+    QString fn = QFileDialog::getOpenFileName(this,tr("Select a File"),"",tr("Images") + " (*.png *.xpm *.jpg *.jpeg *.bmp *.svg)");
 	if(!fn.isEmpty()){
 		item->setIcon(QIcon(fn));
 		replacedIconsOnMenus->insert(item->data(Qt::UserRole).toString(),fn);
@@ -786,6 +817,26 @@ void ConfigDialog::populatePossibleActions(QTreeWidgetItem* parent, const QMenu*
 	}
 }
 
+void ConfigDialog::importDictionary() {
+	QString filename = QFileDialog::getOpenFileName(this, tr("Import Dictionary"), QString(), tr("OpenOffice Dictionary") + " (*.oxt)", 0, QFileDialog::DontResolveSymlinks);
+	if (filename.isNull()) return;
+
+	ConfigManager *config = dynamic_cast<ConfigManager *>(ConfigManagerInterface::getInstance());
+	if (!config) return;
+
+	QString targetDir = config->parseDir("[txs-settings-dir]/dictionaries");
+	targetDir.replace("//", "/");  // resolution of [txs-settings-dir] may lead to duplicate path separators
+	SpellerManager::importDictionary(filename, targetDir);
+
+	if (!config->parseDirList(ui.leDictDir->text()).contains(targetDir)) {
+		if (txsConfirm(tr("The dictionary files have been imported to\n%1.\nHowever this path is not contained in the dictionary path list. Do you want to add it?").arg(targetDir))) {
+			ui.leDictDir->setText(ui.leDictDir->text()+";[txs-settings-dir]/dictionaries");
+		}
+	}
+
+	updateDefaultDictSelection(ui.leDictDir->text(), QFileInfo(filename).baseName()); // TODO: better detection for new dictionary name (.dic filename may be different from .oxt filename)
+}
+
 void ConfigDialog::updateCheckNow() {
 	UpdateChecker::instance()->check(false);
 }
