@@ -1,10 +1,7 @@
 #include "diffoperations.h"
 #include "latexdocument.h"
 #include "smallUsefulFunctions.h"
-DiffOp::DiffOp(){
-	dlh=0;
-	lineWasModified=false;
-}
+DiffOp::DiffOp(): start(0),length(0),type(Insert), lineWasModified(false),dlh(0){}
 
 void diffDocs(LatexDocument *doc,LatexDocument *doc2,bool dontAddLines){
 	
@@ -221,10 +218,11 @@ void diffDocs(LatexDocument *doc,LatexDocument *doc2,bool dontAddLines){
 			lineNr+=elem.text.count("\n");
 		}
 	}
+    doc->mayHaveDiffMarkers = true;
 }
 
 void diffRemoveMarkers(LatexDocument *doc,bool theirs){
-	if(!doc)
+    if(!doc || !doc->mayHaveDiffMarkers)
 		return;
 	
 	QDocumentCursor cur(doc);
@@ -284,6 +282,7 @@ void diffRemoveMarkers(LatexDocument *doc,bool theirs){
 			doc->line(i).removeCookie(QDocumentLine::DIFF_LIST_COOCKIE);
 		}
 	}
+    doc->mayHaveDiffMarkers = false;
 }
 
 void diffChange(LatexDocument *doc,int ln,int col,bool theirs){
