@@ -12,7 +12,7 @@
 //FileNamePair::FileNamePair(const QString& rel):relative(rel){};
 FileNamePair::FileNamePair(const QString& rel, const QString& abs):relative(rel),absolute(abs){};
 
-LatexDocument::LatexDocument(QObject *parent):QDocument(parent),edView(0),mAppendixLine(0)
+LatexDocument::LatexDocument(QObject *parent):QDocument(parent),remeberAutoReload(false),mayHaveDiffMarkers(false),edView(0),mAppendixLine(0)
 {
 	baseStructure = new StructureEntry(this,StructureEntry::SE_DOCUMENT_ROOT);
 	magicCommentList = new StructureEntry(this, StructureEntry::SE_OVERVIEW);
@@ -237,7 +237,7 @@ void LatexDocument::patchStructureRemoval(QDocumentLineHandle* dlh) {
 	}
 
 	LatexParser& latexParser = LatexParser::getInstance();
-	QVector<StructureEntry*> parent_level(latexParser.structureCommands.count());
+	QVector<StructureEntry*> parent_level(latexParser.structureDepth());
 	
 	QList<StructureEntry*> ls;
 	mergeStructure(baseStructure, parent_level, ls, linenr, 1);
@@ -418,27 +418,18 @@ void LatexDocument::patchStructure(int linenr, int count) {
 		s=curLine;
 		l=s.indexOf("% !TeX",0,Qt::CaseInsensitive);
 		if (l>=0) {
-			s=s.mid(l+6,s.length());
-			bool reuse=false;
-			StructureEntry *newMagicComment;
-			if(MapOfMagicComments.contains(dlh)){
-				newMagicComment=MapOfMagicComments.value(dlh);
-				newMagicComment->type=StructureEntry::SE_MAGICCOMMENT;
-				MapOfMagicComments.remove(dlh,newMagicComment);
-				reuse=true;
-			}else{
-				newMagicComment=new StructureEntry(this, StructureEntry::SE_MAGICCOMMENT);
-			}
+			addMagicComment(s.mid(l+6).trimmed(), i, MapOfMagicComments, iter_magicComment);
+		}
+		l=s.indexOf("% !BIB",0,Qt::CaseInsensitive);
+		if (l>=0) {
+			// workaround to also support "% !BIB program = biber" syntax used by TeXShop and TeXWorks
+			s=s.mid(l+6).trimmed();
 			QString name;
 			QString val;
 			splitMagicComment(s, name, val);
-			
-			parseMagicComment(name, val, newMagicComment);
-			newMagicComment->title=s;
-			newMagicComment->setLine(line(i).handle(), i);
-			newMagicComment->parent=magicCommentList;
-			if(!reuse) emit addElement(magicCommentList,magicCommentList->children.size()); //todo: why here but not in label?
-			iter_magicComment.insert(newMagicComment);
+			if ((name=="TS-program" || name=="program") && (val=="biber" || val=="bibtex")) {
+				addMagicComment(QString("program:bibliography = txs:///%1").arg(val), i, MapOfMagicComments, iter_magicComment);
+			}
 		}
 		////Ref
 		//for reference counting (can be placed in command options as well ...
@@ -499,17 +490,39 @@ void LatexDocument::patchStructure(int linenr, int count) {
 			mAppendixLine=0;
 		}
 		//let %\include be processed
-		if(curLine.startsWith("%\\include")||curLine.startsWith("%\\input")){
+		if(curLine.startsWith("%\\include")||curLine.startsWith("%\\input")||curLine.startsWith("%\\import")){
 			curLine.replace(0,1,' ');
 		}
 		int totalLength=curLine.length();
-        while(findCommandWithArg(curLine,cmd,name,arg,remainder,optionStart)){
+        QString option;
+        while(findCommandWithArg(curLine,cmd,name,arg,remainder,optionStart,option)){
 			//update offset
 			//store optional arguments []
 			
 			//copy remainder to curLine for next round
 			curLine=remainder;
 			int offset=totalLength-curLine.length(); //TODO?? (line was commented out, with todo before)
+			
+			if (latexParser.possibleCommands["%todo"].contains(cmd)) {
+				bool reuse=false;
+				StructureEntry *newTodo;
+				if(MapOfTodo.contains(dlh)){
+					newTodo=MapOfTodo.value(dlh);
+					//parent->add(newTodo);
+					newTodo->type=StructureEntry::SE_TODO;
+					MapOfTodo.remove(dlh,newTodo);
+					reuse=true;
+				}else{
+					newTodo=new StructureEntry(this, StructureEntry::SE_TODO);
+				}
+				newTodo->title=name;
+				newTodo->setLine(line(i).handle(), i);
+				newTodo->parent=todoList;
+				if(!reuse) emit addElement(todoList,todoList->children.size()); //todo: why here but not in label?
+				iter_todo.insert(newTodo);
+			}
+			
+			
 			//// newcommand ////
 			//TODO: handle optional arguments
 			if (latexParser.possibleCommands["%definition"].contains(cmd)||ltxCommands.possibleCommands["%definition"].contains(cmd)) {
@@ -524,12 +537,11 @@ void LatexDocument::patchStructure(int linenr, int count) {
                     }
                 }
 				completerNeedsUpdate=true;
-				QRegExp rx("^\\s*\\[(\\d+)\\](\\[.+\\])?");
-				int options=0;
+				QRegExp rx("^\\s*(\\[.+\\])?");
+				int options=arg.toInt();
 				int def=0;
 				if(rx.indexIn(remainder)>-1){
-					options=rx.cap(1).toInt(); //returns 0 if conversion fails
-					if(!rx.cap(2).isEmpty())
+					if(!rx.cap(1).isEmpty())
 						def=1;
 				}
 				ltxCommands.possibleCommands["user"].insert(name);
@@ -562,21 +574,38 @@ void LatexDocument::patchStructure(int linenr, int count) {
 				continue;
 			}
 			// special treatment \def
-			if (cmd=="\\def") {
+			if (cmd=="\\def" || cmd == "\\gdef" || cmd == "\\edef" || cmd == "\\xdef") {
 				completerNeedsUpdate=true;
-				QRegExp rx("(\\\\\\w+)\\s*(#\\d+)?");
+				QRegExp rx("(\\\\\\w+)\\s*([^{%]*)");
 				if(rx.indexIn(remainder)>-1){
 					QString name=rx.cap(1);
 					QString optionStr=rx.cap(2);
 					//qDebug()<< name << ":"<< optionStr;
-					int options=optionStr.mid(1).toInt(); //returns 0 if conversion fails
 					ltxCommands.possibleCommands["user"].insert(name);
-                    if(!removedUserCommands.removeAll(name)){
-                        addedUserCommands << name;
-                    }
-					for (int j=0; j<options; j++) {
-						if (j==0) name.append("{%<arg1%|%>}");
-						else name.append(QString("{%<arg%1%>}").arg(j+1));
+					if(!removedUserCommands.removeAll(name)) addedUserCommands << name;
+					optionStr = optionStr.trimmed();
+					if (optionStr.length()) {
+						int lastArg = optionStr[optionStr.length()-1].toLatin1() - '0';
+						if (optionStr.length() == lastArg * 2) { //#1#2#3...
+							for (int j=1; j<=lastArg; j++)
+								if (j==1) name.append("{%<arg1%|%>}");
+								else name.append(QString("{%<arg%1%>}").arg(j));
+						} else {
+							QStringList args = optionStr.split('#'); //#1;#2#3:#4 => ["",1;,2,3:,4]
+							args.removeAt(0);
+							bool hadSeparator = true;
+							for (int i=0;i<args.length();i++) {
+								if (args[i].length() == 0) continue; //invalid
+								bool hasSeparator = (args[i].length() != 1); //only single digit variables allowed. last arg also needs a sep
+								if (!hadSeparator || !hasSeparator)
+									args[i] = "{%<arg"+args[i][0]+"%>}" + args[i].mid(1);
+								else
+									args[i] = "%<arg"+args[i][0]+"%>" + args[i].mid(1); //no need to use {} for arguments that are separated anyways
+								hadSeparator  = hasSeparator;
+							}
+							name.append(" ");
+							name.append(args.join(""));
+						}
 					}
 					mUserCommandList.insert(line(i).handle(),name);
 					// remove obsolete Overlays (maybe this can be refined
@@ -651,12 +680,30 @@ void LatexDocument::patchStructure(int linenr, int count) {
 			if (latexParser.possibleCommands["%usepackage"].contains(cmd)) {
 				completerNeedsUpdate=true;
 				QStringList packagesHelper=name.split(",");
-				if(cmd.endsWith("theme")){ // special treatment for  \usetheme
+
+                if(cmd.endsWith("theme")){ // special treatment for  \usetheme
 					QString preambel=cmd;
 					preambel.remove(0,4);
 					preambel.prepend("beamer");
 					packagesHelper.replaceInStrings(QRegExp("^"),preambel);
 				}
+
+                if(cmd=="\\documentclass"){
+                    //special treatment for documentclass, especially for the class options
+                    // at the moment a change here soes not automatically lead to an update of corresponding definitions, here babel
+                    mClassOptions=option;
+                }
+
+                if(name=="babel"){
+                    //special treatment for babel
+                    if(option.isEmpty()){
+                        option=mClassOptions;
+                    }
+                    if(!option.isEmpty()){
+                        packagesHelper << option.split(",");
+                    }
+                }
+
 				QStringList packages;
                 foreach(QString elem,packagesHelper){
                     elem=elem.simplified();
@@ -675,10 +722,11 @@ void LatexDocument::patchStructure(int linenr, int count) {
 			}
 			//// bibliography ////
 			if (latexParser.possibleCommands["%bibliography"].contains(cmd)) {
+				QStringList additionalBibPaths = ConfigManagerInterface::getInstance()->getOption("Files/Bib Paths").toString().split(getPathListSeparator());
 				QStringList bibs=name.split(',',QString::SkipEmptyParts);
 				//add new bibs and set bibTeXFilesNeedsUpdate if there was any change
 				foreach(const QString& elem,bibs){ //latex doesn't seem to allow any spaces in file names
-					mMentionedBibTeXFiles.insert(line(i).handle(),FileNamePair(elem,getAbsoluteFilePath(elem,"bib")));
+					mMentionedBibTeXFiles.insert(line(i).handle(),FileNamePair(elem,getAbsoluteFilePath(elem,"bib",additionalBibPaths)));
 					if (oldBibs.removeAll(elem) == 0)
 						bibTeXFilesNeedsUpdate = true;
 				}
@@ -703,7 +751,6 @@ void LatexDocument::patchStructure(int linenr, int count) {
 			//// beamer blocks ////
 			
 			if (cmd=="\\begin" && name=="block") {
-				QString s=extractSectionName(remainder,true);
 				StructureEntry *newBlock;
 				if(MapOfBlock.contains(dlh)){
 					newBlock=MapOfBlock.value(dlh);
@@ -712,19 +759,20 @@ void LatexDocument::patchStructure(int linenr, int count) {
 				}else{
 					newBlock=new StructureEntry(this, StructureEntry::SE_BLOCK);
 				}
-				newBlock->title=s;
+				newBlock->title=arg;
 				newBlock->setLine(line(i).handle(), i);
 				newBlock->parent=blockList;
 				iter_block.insert(newBlock);
 				continue;
 			}
 			
-			//// include,input ////
-			//static const QStringList inputTokens = QStringList() << "\\input" << "\\include";
+			//// include,input,import ////
+			//static const QStringList inputTokens = QStringList() << "\\input" << "\\include" << "\\import";
 			
 			if (latexParser.possibleCommands["%include"].contains(cmd) && !isDefinitionArgument(name)) {
 				StructureEntry *newInclude=new StructureEntry(this, StructureEntry::SE_INCLUDE);
-				newInclude->level = parent && !parent->indentIncludesInStructure ? 0 : latexParser.structureCommands.count() - 1;
+				newInclude->level = parent && !parent->indentIncludesInStructure ? 0 : latexParser.structureDepth() - 1;
+				name = removeQuote(name);
 				newInclude->title=name;
                 QString fname=findFileName(name);
                 removedIncludes.removeAll(fname);
@@ -743,16 +791,41 @@ void LatexDocument::patchStructure(int linenr, int count) {
                 updateSyntaxCheck=true;
 				continue;
 			}
+			
+			if (latexParser.possibleCommands["%import"].contains(cmd) && !isDefinitionArgument(name)) {
+				StructureEntry *newInclude=new StructureEntry(this, StructureEntry::SE_INCLUDE);
+				newInclude->level = parent && !parent->indentIncludesInStructure ? 0 : latexParser.structureDepth() - 1;
+                QFileInfo fi(name,arg);
+                QString file = fi.filePath();
+				newInclude->title = file;
+                QString fname=findFileName(file);
+                removedIncludes.removeAll(fname);
+                mIncludedFilesList.insert(line(i).handle(),fname);
+				LatexDocument* dc=parent->findDocumentFromName(fname);
+                if(dc)	dc->setMasterDocument(this,recheckLabels);
+                else {
+                    lstFilesToLoad << fname;
+                    //parent->addDocToLoad(fname);
+                }
+
+                newInclude->valid = !fname.isEmpty();
+				newInclude->setLine(line(i).handle(), i);
+				newInclude->columnNumber = offset;
+				flatStructure << newInclude;
+                updateSyntaxCheck=true;
+				continue;
+			}
+
 			//// all sections ////
 			if(cmd.endsWith("*"))
 				cmd=cmd.left(cmd.length()-1);
-			int header=latexParser.structureCommands.indexOf(cmd);
-			if (header>-1) {
+			int level = latexParser.structureCommandLevel(cmd);
+			if (level>-1) {
 				StructureEntry *newSection = new StructureEntry(this,StructureEntry::SE_SECTION);
 				if(mAppendixLine &&indexOf(mAppendixLine)<i) newSection->appendix=true;
 				else newSection->appendix=false;
 				newSection->title=parseTexOrPDFString(name);
-				newSection->level=header;
+				newSection->level=level;
 				newSection->setLine(line(i).handle(), i);
 				newSection->columnNumber = offset;
 				flatStructure << newSection;
@@ -767,7 +840,7 @@ void LatexDocument::patchStructure(int linenr, int count) {
             parent->updateMasterSlaveRelations(this);
         }
 	}//for each line handle
-	QVector<StructureEntry*> parent_level(latexParser.structureCommands.count());
+	QVector<StructureEntry*> parent_level(latexParser.structureDepth());
     if(!isHidden()){
         mergeStructure(baseStructure, parent_level, flatStructure, linenr, count);
 
@@ -1021,8 +1094,8 @@ QMultiHash<QDocumentLineHandle*,int> LatexDocument::getRefs(const QString& name)
 
 void LatexDocument::setMasterDocument(LatexDocument* doc,bool recheck){
 	masterDocument=doc;
-	QList<LatexDocument *>listOfDocs=getListOfDocs();
     if(recheck){
+        QList<LatexDocument *>listOfDocs=getListOfDocs();
         foreach(LatexDocument *elem,listOfDocs){
             elem->recheckRefsLabels();
         }
@@ -1267,10 +1340,12 @@ StructureEntry* StructureEntryIterator::next(){
 LatexDocumentsModel::LatexDocumentsModel(LatexDocuments& docs):documents(docs),
   iconDocument(":/images/doc.png"), iconMasterDocument(":/images/masterdoc.png"), iconBibTeX(":/images/bibtex.png"), iconInclude(":/images/include.png"),
   iconWarning(getRealIconCached("warning")), m_singleMode(false){
-  mHighlightIndex=QModelIndex();
-  iconSection.resize(LatexParser::getInstance().structureCommands.count());
-  for (int i=0;i<LatexParser::getInstance().structureCommands.count();i++)
-    iconSection[i]=getRealIconCached(LatexParser::getInstance().structureCommands[i].mid(1));
+	mHighlightIndex=QModelIndex();
+
+	QStringList structureIconNames = QStringList() << "part" << "chapter" << "section" << "subsection" << "subsubsection" << "paragraph";
+	iconSection.resize(structureIconNames.length());
+	for (int i=0; i<structureIconNames.length(); i++)
+		iconSection[i] = getRealIconCached(structureIconNames[i]);
 }
 Qt::ItemFlags LatexDocumentsModel::flags ( const QModelIndex & index ) const{
 	if (index.isValid()) return Qt::ItemIsEnabled|Qt::ItemIsSelectable;
@@ -1428,12 +1503,12 @@ QModelIndex LatexDocumentsModel::parent ( const QModelIndex & index ) const{
 	if (!entry) return QModelIndex();
 	if (!entry->parent) return QModelIndex();
 	
-	if(entry->level>LatexParser::getInstance().structureCommands.count() || entry->level<0){
+	if(entry->level>LatexParser::getInstance().structureDepth() || entry->level<0){
 		entry->debugPrint("Structure broken!");
 		//qDebug("Title %s",qPrintable(entry->title));
 		return QModelIndex();
 	}
-	if(entry->parent->level>LatexParser::getInstance().structureCommands.count() || entry->parent->level<0){
+	if(entry->parent->level>LatexParser::getInstance().structureDepth() || entry->parent->level<0){
 		entry->debugPrint("Structure broken (b)!");
 		//qDebug("Title %s",qPrintable(entry->title));
 		return QModelIndex();
@@ -1602,6 +1677,14 @@ LatexDocuments::~LatexDocuments(){
 void LatexDocuments::addDocument(LatexDocument* document,bool hidden){
     if(hidden){
         hiddenDocuments.append(document);
+        LatexEditorView *edView=document->getEditorView();
+        if (edView) {
+            QEditor* ed=edView->getEditor();
+            if(ed){
+                document->remeberAutoReload=ed->silentReloadOnExternalChanges();
+                ed->setSilentReloadOnExternalChanges(true);
+            }
+        }
     }else{
         documents.append(document);
     }
@@ -1625,7 +1708,7 @@ void LatexDocuments::addDocument(LatexDocument* document,bool hidden){
     if(!hidden)
         model->structureUpdated(document,0);
 }
-void LatexDocuments::deleteDocument(LatexDocument* document,bool hidden){
+void LatexDocuments::deleteDocument(LatexDocument* document,bool hidden,bool purge){
     if(!hidden)
         emit aboutToDeleteDocument(document);
 	LatexEditorView *view=document->getEditorView();
@@ -1634,6 +1717,35 @@ void LatexDocuments::deleteDocument(LatexDocument* document,bool hidden){
 	if (document!=masterDocument) {
 		// get list of all affected documents
 		QList<LatexDocument*> lstOfDocs=document->getListOfDocs();
+        // special treatment to remove document in purge mode (hidden doc was deleted on disc)
+        if(purge){
+            LatexDocument *master=document->getTopMasterDocument();
+            hiddenDocuments.removeAll(document);
+            //update children (connection to parents is severed)
+            foreach(LatexDocument *elem,lstOfDocs){
+                if(elem->getMasterDocument()==document){
+                    if(elem->isHidden())
+                        deleteDocument(elem,true,true);
+                    else
+                        elem->setMasterDocument(0);
+                }
+            }
+            delete document;
+            // update parents
+            lstOfDocs=master->getListOfDocs();
+            int n=0;
+            foreach(LatexDocument* elem,lstOfDocs){
+                if(!elem->isHidden()){
+                    n++;
+                    break;
+                }
+            }
+            if(n==0)
+                deleteDocument(master,true,true);
+            else
+                updateMasterSlaveRelations(master,true,true);
+            return;
+        }
         // count open related (child/parent) documents
         int n=0;
         foreach(LatexDocument* elem,lstOfDocs){
@@ -1646,6 +1758,14 @@ void LatexDocuments::deleteDocument(LatexDocument* document,bool hidden){
         }
         if(n>1){ // at least one related document will be open after removal
             hiddenDocuments.append(document);
+            LatexEditorView *edView=document->getEditorView();
+            if (edView) {
+                QEditor* ed=edView->getEditor();
+                if(ed){
+                    document->remeberAutoReload=ed->silentReloadOnExternalChanges();
+                    ed->setSilentReloadOnExternalChanges(true);
+                }
+            }
         }else{
             /*
             // set document.masterdocument = 0
@@ -1706,6 +1826,13 @@ void LatexDocuments::deleteDocument(LatexDocument* document,bool hidden){
 			currentDocument=0;
 	}
 }
+
+void LatexDocuments::requestedClose(){
+    QEditor *editor = qobject_cast<QEditor*>(sender());
+    LatexDocument *doc=qobject_cast<LatexDocument*>(editor->document());
+    deleteDocument(doc,true,true);
+}
+
 void LatexDocuments::setMasterDocument(LatexDocument* document){
 	if (document==masterDocument) return;
 	if (masterDocument!=0 && masterDocument->getEditorView()==0){
@@ -1779,23 +1906,9 @@ QString LatexDocuments::getTemporaryCompileFileName() const {
 	return "";
 }
 
-QString LatexDocuments::getAbsoluteFilePath(const QString & relName, const QString &baseDir, const QString &extension) const {
-    QString s=relName;
-    QString ext = extension;
-    if (!ext.isEmpty() && !ext.startsWith(".")) ext = "." + ext;
-    if (!s.endsWith(ext,Qt::CaseInsensitive)) s+=ext;
-    QFileInfo fi(s);
-    if (!fi.isRelative()) return s;
-    if (baseDir.isEmpty()) return s; //what else can we do?
-    QString compilePath=QFileInfo(baseDir).absolutePath();
-    if (!compilePath.endsWith("\\") && !compilePath.endsWith("/"))
-        compilePath+=QDir::separator();
-    return  compilePath+s;
-}
-
-QString LatexDocuments::getAbsoluteFilePath(const QString & relName, const QString &extension) const {
-	QString compileFileName=getTemporaryCompileFileName();
-    return  getAbsoluteFilePath(relName,compileFileName,extension);
+QString LatexDocuments::getAbsoluteFilePath(const QString & relName, const QString &extension, const QStringList &additionalSearchPaths) const {
+	if (!currentDocument) return relName;
+	return currentDocument->getAbsoluteFilePath(relName, extension, additionalSearchPaths);
 }
 
 LatexDocument* LatexDocuments::findDocumentFromName(const QString& fileName) const {
@@ -1824,7 +1937,7 @@ LatexDocument* LatexDocuments::findDocument(const QString& fileName, bool checkT
 	QString fnorm = fileName;
 	fnorm.replace("/",QDir::separator()).replace("\\",QDir::separator());
 	//fast check for absolute file names
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	Qt::CaseSensitivity cs = Qt::CaseInsensitive;
 #else
 	Qt::CaseSensitivity cs = Qt::CaseSensitive;
@@ -1863,9 +1976,7 @@ LatexDocument* LatexDocuments::findDocument(const QString& fileName, bool checkT
 	return 0;
 }
 void LatexDocuments::settingsRead(){
-	model->iconSection.resize(LatexParser::getInstance().structureCommands.count());
-	for (int i=0;i<LatexParser::getInstance().structureCommands.count();i++)
-        model->iconSection[i]=getRealIcon(LatexParser::getInstance().structureCommands[i].mid(1));
+	return; // currently unused
 }
 bool LatexDocuments::singleMode() const {
 	return !masterDocument;
@@ -1873,18 +1984,19 @@ bool LatexDocuments::singleMode() const {
 
 void LatexDocuments::updateBibFiles(bool updateFiles){
   mentionedBibTeXFiles.clear();
+  QStringList additionalBibPaths = ConfigManagerInterface::getInstance()->getOption("Files/Bib Paths").toString().split(getPathListSeparator());
   foreach (LatexDocument* doc, getDocuments() ) {
     if(updateFiles){
       QMultiHash<QDocumentLineHandle*,FileNamePair>::iterator it = doc->mentionedBibTeXFiles().begin();
       QMultiHash<QDocumentLineHandle*,FileNamePair>::iterator itend = doc->mentionedBibTeXFiles().end();
-      for (; it != itend; ++it){
-        if (it.value().absolute.isEmpty()) it.value().absolute = getAbsoluteFilePath(it.value().relative,".bib").replace(QDir::separator(), "/"); //store absolute
+	  for (; it != itend; ++it){
+		it.value().absolute = getAbsoluteFilePath(it.value().relative,".bib",additionalBibPaths).replace(QDir::separator(), "/"); // update absolute path
         mentionedBibTeXFiles << it.value().absolute;
       }
     }
   }
 
-  bool changed=false;
+  //bool changed=false;
   if(updateFiles){
     for (int i=0; i<mentionedBibTeXFiles.count();i++){
       QString &fileName=mentionedBibTeXFiles[i];
@@ -1893,8 +2005,11 @@ void LatexDocuments::updateBibFiles(bool updateFiles){
       if (!bibTeXFiles.contains(fileName))
         bibTeXFiles.insert(fileName,BibTeXFileInfo());
       BibTeXFileInfo& bibTex=bibTeXFiles[mentionedBibTeXFiles[i]];
-      if (bibTex.loadIfModified(fileName))
-        changed = true;
+
+      bibTex.loadIfModified(fileName);
+
+      /*if (bibTex.loadIfModified(fileName))
+        changed = true;*/
       if (bibTex.ids.empty() && !bibTex.linksTo.isEmpty())
         //handle obscure bib tex feature, a just line containing "link fileName"
         mentionedBibTeXFiles.append(bibTex.linksTo);
@@ -2137,6 +2252,38 @@ bool LatexDocument::splitMagicComment(const QString &comment, QString &name, QSt
 	return true;
 }
 
+/*!
+  Used by the parser to add a magic comment
+
+\a text is the comment without the leading "! TeX" declaration. e.g. "spellcheck = DE-de"
+\a lineNr - line number of the magic comment
+\a MapOfMagicComments - MutliHashTable of all magic comments
+\a iter_magicComment - iterator over all magic comments placed at the last entry before line
+  */
+void LatexDocument::addMagicComment(const QString &text, int lineNr, QMultiHash<QDocumentLineHandle*,StructureEntry*> &MapOfMagicComments, QMutableListIterator<StructureEntry*>& iter_magicComment) {
+	bool reuse=false;
+	StructureEntry *newMagicComment;
+	QDocumentLineHandle *dlh = line(lineNr).handle();
+	if(MapOfMagicComments.contains(dlh)){
+		newMagicComment=MapOfMagicComments.value(dlh);
+		newMagicComment->type=StructureEntry::SE_MAGICCOMMENT;
+		MapOfMagicComments.remove(dlh,newMagicComment);
+		reuse=true;
+	}else{
+		newMagicComment=new StructureEntry(this, StructureEntry::SE_MAGICCOMMENT);
+	}
+	QString name;
+	QString val;
+	splitMagicComment(text, name, val);
+
+	parseMagicComment(name, val, newMagicComment);
+	newMagicComment->title=text;
+	newMagicComment->setLine(dlh, lineNr);
+	newMagicComment->parent=magicCommentList;
+	if(!reuse) emit addElement(magicCommentList,magicCommentList->children.size()); //todo: why here but not in label?
+	iter_magicComment.insert(newMagicComment);
+}
+
 /*!
   Formats the StructureEntry and modifies the document according to the MagicComment contents
   */
@@ -2227,6 +2374,38 @@ bool LatexDocument::fileExits(QString fname){
 	return exist;
 }
 
+/*
+ * A line snapshot is a list of DocumentLineHandles at a given time.
+ * For example, this is used to reconstruct the line number at latex compile time
+ * allowing syncing from PDF to the correct source line also after altering the source document
+ */
+void LatexDocument::saveLineSnapshot() {
+	foreach (QDocumentLineHandle *dlh, mLineSnapshot) {
+		dlh->deref();
+	}
+	mLineSnapshot.clear();
+#if (QT_VERSION >= 0x040700)
+	mLineSnapshot.reserve(lineCount());
+#endif
+	QDocumentConstIterator it = begin(), e = end();
+	while (it != e) {
+		mLineSnapshot.append(*it);
+		(*it)->ref();
+		it++;
+	}
+}
+
+// get the line with given lineNumber (0-based) from the snapshot
+QDocumentLine LatexDocument::lineFromLineSnapshot(int lineNumber) {
+	if (lineNumber < 0 || lineNumber >= mLineSnapshot.count()) return QDocumentLine();
+	return QDocumentLine(mLineSnapshot.at(lineNumber));
+}
+
+// returns the 0-based number of the line in the snapshot, or -1 if line is not in the snapshot
+int LatexDocument::lineToLineSnapshotLineNumber(const QDocumentLine &line) {
+	return mLineSnapshot.indexOf(line.handle());
+}
+
 QString LatexDocument::findFileName(QString fname){
 	QString curPath=ensureTrailingDirSeparator(getFileInfo().absolutePath());
 	QString result;
@@ -2252,7 +2431,7 @@ void LatexDocuments::bibTeXFilesNeedUpdate(){
 	bibTeXFilesModified=true;
 }
 
-void LatexDocuments::updateMasterSlaveRelations(LatexDocument *doc, bool recheckRefs){
+void LatexDocuments::updateMasterSlaveRelations(LatexDocument *doc, bool recheckRefs,bool updateCompleterNow){
 	//update Master/Child relations
 	//remove old settings ...
     doc->setMasterDocument(0,false);
@@ -2291,6 +2470,9 @@ void LatexDocuments::updateMasterSlaveRelations(LatexDocument *doc, bool recheck
 	//recheck references
     if(recheckRefs)
         doc->recheckRefsLabels();
+
+    if(updateCompleterNow)
+        doc->emitUpdateCompleter();
 }
 
 const LatexDocument* LatexDocument::getTopMasterDocument(QSet<const LatexDocument*> *visitedDocs) const {
@@ -2379,7 +2561,7 @@ bool LatexDocument::updateCompletionFiles(bool forceUpdate,bool forceLabelUpdate
 	//patch lines for new commands (ref,def, etc)
 	LatexParser& latexParser = LatexParser::getInstance();
 	QStringList categories;
-    categories<< "%ref" << "%label" << "%definition" << "%cite" << "%citeExtended" << "%citeExtendedCommand" << "%usepackage" << "%graphics" << "%file" << "%bibliography" << "%include";
+	categories<< "%ref" << "%label" << "%definition" << "%cite" << "%citeExtended" << "%citeExtendedCommand" << "%usepackage" << "%graphics" << "%file" << "%bibliography" << "%include" << "%todo";
 	QStringList newCmds;
 	foreach(const QString elem,categories){
 		QStringList cmds=ltxCommands.possibleCommands[elem].values();
@@ -2411,6 +2593,10 @@ bool LatexDocument::updateCompletionFiles(bool forceUpdate,bool forceLabelUpdate
     return false;
 }
 
+void LatexDocument::emitUpdateCompleter(){
+    emit updateCompleter();
+}
+
 void LatexDocument::gatherCompletionFiles(QStringList &files,QStringList &loadedFiles,LatexPackage &pck){
 	LatexPackage zw;
 	LatexCompleterConfig *completerConfig=edView->getCompleter()->getConfig();
@@ -2430,7 +2616,13 @@ void LatexDocument::gatherCompletionFiles(QStringList &files,QStringList &loaded
 			}
 		}
 		if(zw.packageName=="<notFound>"){
-			emit importPackage(elem);
+            QString name=elem;
+            LatexDocument *masterDoc=getTopMasterDocument();
+            if(masterDoc){
+                QString fn=masterDoc->getFileInfo().absolutePath();
+                name+="/"+fn;
+            }
+            emit importPackage(name);
 		} else {
 			pck.unite(zw);
 			loadedFiles.append(elem);
@@ -2545,20 +2737,18 @@ LatexDocument *LatexDocuments::getMasterDocumentForDoc(LatexDocument *doc) const
 	return current->getTopMasterDocument();
 }
 
-QString LatexDocument::getAbsoluteFilePath(const QString & relName, const QString &extension) const {
-	QString s=relName;
-	QString ext = extension;
-	if (!ext.isEmpty() && !ext.startsWith(".")) ext = "." + ext;
-	if (!s.endsWith(ext,Qt::CaseInsensitive)) s+=ext;
-	QFileInfo fi(s);
-	if (!fi.isRelative()) return s;
-	const LatexDocument *masterDoc=getTopMasterDocument();
-	QString compileFileName=masterDoc->getFileName();
-	if (compileFileName.isEmpty()) return s; //what else can we do?
-	QString compilePath=QFileInfo(compileFileName).absolutePath();
-	if (!compilePath.endsWith("\\") && !compilePath.endsWith("/"))
-		compilePath+=QDir::separator();
-	return  compilePath+s;
+QString LatexDocument::getAbsoluteFilePath(const QString & relName, const QString &extension, const QStringList &additionalSearchPaths) const {
+	QStringList searchPaths;
+	const LatexDocument* realMaster = getTopMasterDocument();
+	QString compileFileName = realMaster->getFileName();
+	if (compileFileName.isEmpty()) compileFileName = realMaster->getTemporaryFileName();
+	QString fallbackPath;
+	if (!compileFileName.isEmpty()) {
+		fallbackPath = QFileInfo(compileFileName).absolutePath(); //when the file does not exist, resolve it relative to document (e.g. to create it there)
+		searchPaths << fallbackPath;
+	}
+	searchPaths << additionalSearchPaths;
+	return findAbsoluteFilePath(relName, extension, searchPaths, fallbackPath);
 }
 
 void LatexDocuments::lineGrammarChecked(const void* doc,const void* line,int lineNr, const QList<GrammarError>& errors){
