@@ -70,6 +70,8 @@ public:
 	Q_PROPERTY(QEditor* editor READ getEditor);
 	Q_PROPERTY(LatexDocument* document READ getDocument);     //<- semicolon necessary due to qt bug 22992
 	
+	LatexEditorViewConfig * getConfig() { return config; }
+
 	Q_INVOKABLE	QString displayName() const;
 
 	//  FindWidget *findwidget;
@@ -134,6 +136,15 @@ public:
         latexPackageList=lst;
     }
 
+    LatexParser lp;
+
+    QString getSearchText();
+    QString getReplaceText();
+    bool getSearchIsCase();
+    bool getSearchIsRegExp();
+    bool getSearchIsWords();
+
+
 private:
 	QAction *lineNumberPanelAction, *lineMarkPanelAction, *lineFoldPanelAction, *lineChangePanelAction, 
 	*statusPanelAction, *searchReplacePanelAction, *gotoLinePanelAction;
@@ -180,7 +191,8 @@ private:
 
 	static QStringList checkedLanguages; // languages for online checking
 
-	
+    QMap<QString,QString> mReplacementList;
+
 private slots:
 	void requestCitation(); //emits needCitation with selected text
 	void openExternalFile();
@@ -229,12 +241,15 @@ public slots:
 	void checkForLinkOverlay(QDocumentCursor cursor);
 	bool hasLinkOverlay() const { return linkOverlay.isValid(); }
 	const LinkOverlay & getLinkOverlay() const { return linkOverlay; }
+
+    void emitColonTyped() {emit colonTyped();}
+
 private:
 	void setLinkOverlay(const LinkOverlay &overlay);
 	void removeLinkOverlay();
 	bool isNonTextFormat(int format);
 	QString extractMath(QDocumentCursor cursor);
-	void showMathEnvPreview(QDocumentCursor cursor, QString command, QString environment, QPoint pos);
+	bool showMathEnvPreview(QDocumentCursor cursor, QString command, QString environment, QPoint pos);
 public slots:
 	void temporaryHighlight(QDocumentCursor cur);
 	void removeTemporaryHighlight();
@@ -273,6 +288,8 @@ signals:
 	void linesChanged(QString language, const void * doc, const QList<LineInfo>& lines, int firstLineNr);
 	void searchBibtexSection(QString file,QString bibId);
     void openInternalDocViewer(QString package,QString command="");
+
+    void searchExtendToggled(bool toggled);
 private slots:
 	void lineMarkContextMenuRequested(int lineNumber, QPoint globalPos);
 	void foldContextMenuRequested(int lineNumber, QPoint globalPos);
