@@ -31,14 +31,21 @@ public:
 	bool smoothScrolling;
 	bool hackAutoChoose, hackDisableFixedPitch, hackDisableWidthCache, hackDisableLineCache,hackDisableAccentWorkaround;
 	int hackRenderingMode; //0: normal, 1: qt (missing), 2: single letter
+	bool showPlaceholders;
 	int wordwrap; // 0 off, 1 soft wrap, 2 soft wrap fixed line width, 3 hard wrap fixed line width
 	bool toolTipPreview;
 	bool toolTipHelp;
+	int maxImageTooltipWidth;
 	bool texdocHelpInInternalViewer;
 	bool silentReload;
 	
 	bool autoInsertLRM, visualColumnMode;
 
+	bool overwriteOpeningBracketFollowedByPlaceholder;
+	bool overwriteClosingBracketFollowingPlaceholder;
+	int contextMenuKeyboardModifiers;  // actually this is a Qt::KeyboardModifiers flag (but we don't want to import the whole qt namespace here)
+
+
 	void settingsChanged();
 private:
 	QString lastFontFamily;
