@@ -11,9 +11,17 @@ LatexStyleParser::LatexStyleParser(QObject *parent,QString baseDirName,QString k
     texdefDir=kpsecmd.left(kpsecmd.length()-9);
     QProcess myProc(0);
     //myProc.start(texdefDir+"texdef");
-    myProc.start(texdefDir+"pdflatex -v");
+    myProc.start(texdefDir+"pdflatex --version");
     myProc.waitForFinished();
     texdefMode=(myProc.exitCode()==0);
+    if(texdefMode){
+	QString output=myProc.readAllStandardOutput();
+	output=output.split("\n").first().trimmed();
+	if(output.contains("MiKTeX")){
+	    kpseWhichCmd.chop(9);
+	    kpseWhichCmd.append("findtexmf");
+	}
+    }
 }
 
 void LatexStyleParser::stop(){
@@ -30,6 +38,14 @@ void LatexStyleParser::run(){
 		QString fn=mFiles.dequeue();
 		mFilesLock.unlock();
         QString topPackage="";
+        QString dirName="";
+        if(fn.contains("/")){
+            int i=fn.indexOf("/");
+            if(i>-1){
+                dirName=fn.mid(i+1);
+                fn=fn.left(i);
+            }
+        }
         if(fn.contains('#')){
             QStringList lst=fn.split('#');
             if(!lst.isEmpty())
@@ -40,12 +56,17 @@ void LatexStyleParser::run(){
             topPackage=fn;
             topPackage.chop(4);
         }
-	QString fullName=kpsewhich(fn); // find file
-	if(fullName.isEmpty())
-	    continue;
+        QString fullName=kpsewhich(fn); // find file
+        if(fullName.isEmpty()){
+            if(!dirName.isEmpty()){
+                fullName=kpsewhich(fn,dirName); // find file
+            }
+        }
+        if(fullName.isEmpty())
+            continue;
 
-	QStringList results,parsedPackages;
-	results=readPackage(fullName,parsedPackages); // parse package(s)
+        QStringList results,parsedPackages;
+        results=readPackage(fullName,parsedPackages); // parse package(s)
         results.sort();
 
         if(texdefMode){
@@ -103,41 +124,41 @@ void LatexStyleParser::run(){
             results<<appendList;
         }
 
-		
-		// if included styles call for additional generation, do it.
-		QStringList included=results.filter(QRegExp("#include:.+"));
-		foreach(QString elem,included){
-			elem=elem.mid(9);
-			QString fn=findResourceFile("completion/"+elem+".cwl",false,QStringList(baseDir));
-			if(fn.isEmpty()){
+
+        // if included styles call for additional generation, do it.
+        QStringList included=results.filter(QRegExp("#include:.+"));
+        foreach(QString elem,included){
+            elem=elem.mid(9);
+            QString fn=findResourceFile("completion/"+elem+".cwl",false,QStringList(baseDir));
+            if(fn.isEmpty()){
                 QString hlp=kpsewhich(elem+".sty");
                 if(!hlp.isEmpty()){
                     if(!topPackage.isEmpty())
                         elem=topPackage+"#"+elem+".sty";
                     addFile(elem);
                 }
-			}
-		}
-		
-		// write results
-		if(!results.isEmpty()){
-		    QFileInfo info(fn);
-		    QString baseName=info.completeBaseName();
-		    if(!baseName.isEmpty()){
-			QFile data(baseDir+"/"+baseName+".cwl");
-			if(data.open(QFile::WriteOnly|QFile::Truncate)){
-			    QTextStream out(&data);
-			    out << "# autogenerated by txs\n";
-			    foreach(const QString& elem,results){
-				out << elem << "\n";
-			    }
-			}
-            if(!topPackage.isEmpty() && topPackage!=baseName)
-                baseName=topPackage+"#"+baseName;
-			emit scanCompleted(baseName);
-		    }
-		}
-	}
+            }
+        }
+
+        // write results
+        if(!results.isEmpty()){
+            QFileInfo info(fn);
+            QString baseName=info.completeBaseName();
+            if(!baseName.isEmpty()){
+                QFile data(baseDir+"/"+baseName+".cwl");
+                if(data.open(QFile::WriteOnly|QFile::Truncate)){
+                    QTextStream out(&data);
+                    out << "# autogenerated by txs\n";
+                    foreach(const QString& elem,results){
+                        out << elem << "\n";
+                    }
+                }
+                if(!topPackage.isEmpty() && topPackage!=baseName)
+                    baseName=topPackage+"#"+baseName;
+                emit scanCompleted(baseName);
+            }
+        }
+    }
 }
 
 void LatexStyleParser::addFile(QString filename){
@@ -157,6 +178,7 @@ QStringList LatexStyleParser::readPackage(QString fn,QStringList& parsedPackages
 	parsedPackages<<fn;
         QString line;
         QRegExp rxDef("\\\\def\\s*(\\\\[\\w@]+)\\s*(#\\d+)?");
+        QRegExp rxLet("\\\\let\\s*(\\\\[\\w@]+)");
         QRegExp rxCom("\\\\(newcommand|providecommand)\\*?\\s*\\{(\\\\\\w+)\\}\\s*\\[?(\\d+)?\\]?");
         QRegExp rxCom2("\\\\(newcommand|providecommand)\\*?\\s*(\\\\\\w+)\\s*\\[?(\\d+)?\\]?");
         QRegExp rxEnv("\\\\newenvironment\\s*\\{(\\w+)\\}\\s*\\[?(\\d+)?\\]?");
@@ -167,6 +189,7 @@ QStringList LatexStyleParser::readPackage(QString fn,QStringList& parsedPackages
         QRegExp rxDecMathSym("\\\\DeclareMathSymbol\\s*\\{\\\\(\\w+)\\}");
         QRegExp rxNewLength("\\\\newlength\\s*\\{\\\\(\\w+)\\}");
         QRegExp rxNewCounter("\\\\newcounter\\s*\\{(\\w+)\\}");
+        QRegExp rxLoadClass("\\\\LoadClass\\s*\\{(\\w+)\\}");
         bool inReq=false;
         while(!stream.atEnd()) {
             line = stream.readLine();
@@ -208,6 +231,15 @@ QStringList LatexStyleParser::readPackage(QString fn,QStringList& parsedPackages
                     results << name;
                 continue;
             }
+            if(rxLet.indexIn(line)>-1){
+                QString name=rxLet.cap(1);
+                if(name.contains("@"))
+                    continue;
+                name.append("#S");
+                if(!results.contains(name))
+                    results << name;
+                continue;
+            }
             if(rxCom.indexIn(line)>-1){
                 QString name=rxCom.cap(2);
                 if(name.contains("@"))
@@ -302,18 +334,35 @@ QStringList LatexStyleParser::readPackage(QString fn,QStringList& parsedPackages
                 }
                 inReq=true;
             }
+            if(rxLoadClass.indexIn(line)>-1){
+                QString arg = rxLoadClass.cap(1);
+                if(!arg.isEmpty()){
+                    if(mPackageAliases.contains(arg))
+                        foreach(QString elem,mPackageAliases.values(arg)){
+                            results << "#include:" + elem;
+                        }
+                    else
+                        results << "#include:" + arg;
+                }
+                continue;
+            }
         } // while line
     } // open data
     return results;
 }
 
-QString LatexStyleParser::kpsewhich(QString name){
+QString LatexStyleParser::kpsewhich(QString name,QString dirName){
     if(name.startsWith("."))
 	return "";  // don't check .sty/.cls
     QString fn=name;
     if(!kpseWhichCmd.isEmpty()){
 	QProcess myProc(0);
-	myProc.start(kpseWhichCmd,QStringList(fn));
+    QStringList options;
+    if(!dirName.isEmpty()){
+        options<<"-path="+dirName;
+    }
+    options<<fn;
+    myProc.start(kpseWhichCmd,options);
 	myProc.waitForFinished();
 	if(myProc.exitCode()==0){
 	    fn=myProc.readAllStandardOutput();
@@ -426,7 +475,7 @@ QStringList LatexStyleParser::readPackageTracing(QString fn){
     }
     QStringList args;
     QString result;
-    args<<"-draftmode"<<"-interaction=nonstopmode" << tf->fileName();
+    args<<"-draftmode"<<"-interaction=nonstopmode" << "--disable-installer"<< tf->fileName();
     myProc.setWorkingDirectory(QFileInfo(tf->fileName()).absoluteDir().absolutePath());
     myProc.start(texdefDir+"pdflatex",args);
     args.clear();
