@@ -316,6 +316,13 @@ QScriptValue include(QScriptContext *context, QScriptEngine *engine){
 	}
 	if (macro.startsWith("%SCRIPT")) macro.remove(0, strlen("%SCRIPT"));
 	if (macro.isEmpty()) return engine->undefinedValue();
+
+	QScriptContext *currentContext = engine->currentContext();
+	QScriptContext *parentContext = currentContext->parentContext();
+	if (parentContext != 0) {
+		currentContext->setActivationObject(parentContext->activationObject());
+		currentContext->setThisObject(parentContext->thisObject());
+	}
 	return engine->evaluate(macro);
 }
 
@@ -337,7 +344,7 @@ void TimeoutWrapper::run(){
 	deleteLater();
 }
 
-scriptengine::scriptengine(QObject *parent) : QObject(parent),globalObject(0), m_editor(0),m_allowWrite(false)
+scriptengine::scriptengine(QObject *parent) : QObject(parent),triggerId(-1), globalObject(0), m_editor(0),m_allowWrite(false)
 {
 	engine=new QScriptEngine(this);
 	qScriptRegisterQObjectMetaType<QDocument*>(engine);
