@@ -6,7 +6,7 @@
 const QString CommonEOW=QString("~!#$%^&*()_+{}|:\"\\<>?,./;[]-= \t\n\r`'") + QChar(171) + QChar(187) + QChar(8223) + QChar(8222) + QChar(8221) + QChar(8220) /* <= fancy quotation marks */;
 const QString Punctation="!():\"?,.;-";
 const QString EscapedChars="%&_";
-const QString CharacterAlteringChars="\"";
+const QString CharacterAlteringChars="\"'^`";
 
 LatexParser* LatexParserInstance = 0;
 
@@ -40,18 +40,19 @@ void LatexParser::init(){
     possibleCommands["tabular"]=QSet<QString>::fromList(QStringList() << "&" );
     possibleCommands["array"]=QSet<QString>::fromList(QStringList() << "&" );
     possibleCommands["tabbing"]=QSet<QString>::fromList(QStringList() << "\\<" << "\\>" << "\\=" << "\\+");
-    possibleCommands["normal"]=QSet<QString>::fromList(QStringList() << "\\\\" << "\\-" << "$" << "$$" << "\\$" << "\\#" << "\\{" << "\\}" << "\\S" << "\\'" << "\\`" << "\\^" << "\\=" <<"\\." <<"\\u" <<"\\v" << "\\H" << "\\t" << "\\c" << "\\d" << "\\b" << "\\oe" << "\\OE" << "\\ae" << "\\AE" << "\\aa" << "\\AA" << "\\o" << "\\O" << "\\P" << "\\l" << "\\L" << "\\~" << "\\ " << "\\,");
+    possibleCommands["normal"]=QSet<QString>::fromList(QStringList() << "\\\\" << "\\-" << "$" << "$$" << "\\$" << "\\#" << "\\{" << "\\}" << "\\S" << "\\'" << "\\`" << "\\^" << "\\=" <<"\\." <<"\\u" <<"\\v" << "\\H" << "\\t" << "\\c" << "\\d" << "\\b" << "\\o" << "\\O" << "\\P" << "\\l" << "\\L" << "\\~" << "\\ " << "\\,");
     possibleCommands["math"]=QSet<QString>::fromList(QStringList() << "_" << "^" << "\\$" << "\\#" << "\\{" << "\\}" << "\\S" << "\\," << "\\!" << "\\;" << "\\:" << "\\\\" << "\\ " << "\\|");
-    possibleCommands["%definition"] << "\\newcommand" << "\\renewcommand" << "\\newcommand*" << "\renewcommand*" << "\\providecommand" << "\\DeclareMathOperator" << "\\DeclareMathSymbol" <<"\\newlength" << "\\DeclareRobustCommand" << "\\let";
+	possibleCommands["%definition"] << "\\newcommand" << "\\renewcommand" << "\\newcommand*" << "\renewcommand*" << "\\providecommand" << "\\newlength" << "\\let";
     possibleCommands["%usepackage"] << "\\usepackage" << "\\documentclass";
     possibleCommands["%graphics"] << "\\includegraphics";
     possibleCommands["%bibitem"]<< "\\bibitem";
     possibleCommands["%cite"]  << "\\cite" <<  "\\nptextcite"  ;
     possibleCommands["%label"] << "\\label";
     possibleCommands["%bibliography"] << "\\bibliography";
-    possibleCommands["%file"] << "\\include" << "\\input" << "\\includeonly" << "\\includegraphics" <<"\\bibliographystyle" << "\\bibliography";
+    possibleCommands["%file"] << "\\include" << "\\input" << "\\import" << "\\includeonly" << "\\includegraphics" <<"\\bibliographystyle" << "\\bibliography";
     possibleCommands["%ref"] << "\\ref" << "\\pageref";
     possibleCommands["%include"] << "\\include" << "\\input";
+    possibleCommands["%import"] << "\\import" << "\\importfrom" << "\\subimport" << "\\subimportfrom";
 }
 
 int LatexReader::nextToken(const QString &line,int &index, bool inOption,bool detectMath) {
@@ -103,10 +104,13 @@ int LatexReader::nextToken(const QString &line,int &index, bool inOption,bool de
 			} else if (cur=='\'') {
 				if (singleQuoteChar) break;	 //no word's with two '' => output
 				else singleQuoteChar=true;   //but accept one
-			} else if (cur=='.') {
+            } else if (cur=='.' || cur=='-') {
 				if (i>0 && line.at(i-1).isLetter())
-					i++; //take '.' into word, so that abbreviations, at least German ones, are checked correctly
-				break;
+                    i++; //take '.' or '-' into word, so that abbreviations/hyphenations, at least German ones, are checked correctly
+				if(cur=='-' &&  i<line.size() && line.at(i).isLetter())
+				    ; // continue with composite words
+				else
+				    break;
 			} else if (CommonEOW.indexOf(cur)>=0 && !inOption) {
 				break;
 			} else if(cur=='}' || cur==']') break;
@@ -186,20 +190,89 @@ QList<QPair<QString,QString> > latexToPlainWordReplaceList =
 //	<< QPair<QString, QString> ("\"\"","") redunant
  << QPair<QString, QString> ("\\",""); // eliminating backslash which might remain from accents like \"a ...
 */
-QChar transformCharacter(const QChar& c){
-    switch (c.toLatin1()){
-	case 'a': return QChar(0xE4);
-	case 'o': return QChar(0xF6);
-	case 'u': return QChar(0xFC);
-	case 'A': return QChar(0xC4);
-	case 'O': return QChar(0xD6);
-	case 'U': return QChar(0xDC);
-	case 's': return QChar(0xDF);
-	default: return c;
+QChar transformCharacter(const QChar& c, const QChar& context){
+	switch (context.toLatin1()) {
+	case '"':  // umlaut
+		switch (c.toLatin1()){
+		case 'a': return QChar(0xE4);
+		case 'e': return QChar(0xEB);
+		case 'i': return QChar(0xEF);
+		case 'o': return QChar(0xF6);
+		case 'u': return QChar(0xFC);
+		case 'A': return QChar(0xC4);
+		case 'E': return QChar(0xCB);
+		case 'I': return QChar(0xCF);
+		case 'O': return QChar(0xD6);
+		case 'U': return QChar(0xDC);
+		case 's': return QChar(0xDF);
+		default: return c;
+		}
+	case '`':  // grave
+		switch (c.toLatin1()){
+		case 'a': return QChar(0xE0);
+		case 'e': return QChar(0xE8);
+		case 'i': return QChar(0xEC);
+		case 'o': return QChar(0xF2);
+		case 'u': return QChar(0xF9);
+		case 'A': return QChar(0xC0);
+		case 'E': return QChar(0xC8);
+		case 'I': return QChar(0xCC);
+		case 'O': return QChar(0xD2);
+		case 'U': return QChar(0xD9);
+		default: return c;
+		}
+	case '\'':  // acute
+		switch (c.toLatin1()){
+		case 'a': return QChar(0xE1);
+		case 'e': return QChar(0xE9);
+		case 'i': return QChar(0xED);
+		case 'o': return QChar(0xF3);
+		case 'u': return QChar(0xFA);
+		case 'y': return QChar(0xFD);
+		case 'A': return QChar(0xC1);
+		case 'E': return QChar(0xC9);
+		case 'I': return QChar(0xCD);
+		case 'O': return QChar(0xD3);
+		case 'U': return QChar(0xDA);
+		case 'Y': return QChar(0xDD);
+		default: return c;
+		}
+	case '^':  // circumflex
+		switch (c.toLatin1()){
+		case 'a': return QChar(0xE2);
+		case 'e': return QChar(0xEA);
+		case 'i': return QChar(0xEE);
+		case 'o': return QChar(0xF4);
+		case 'u': return QChar(0xFB);
+		case 'A': return QChar(0xC2);
+		case 'E': return QChar(0xCA);
+		case 'I': return QChar(0xCE);
+		case 'O': return QChar(0xD4);
+		case 'U': return QChar(0xDB);
+		default: return c;
+		}
+	case '~':  // tilde
+		switch (c.toLatin1()){
+		case 'a': return QChar(0xE3);
+		case 'n': return QChar(0xF1);
+		case 'o': return QChar(0xF5);
+		case 'A': return QChar(0xC3);
+		case 'N': return QChar(0xD1);
+		case 'O': return QChar(0xD5);
+		default: return c;
+		}
+	case 'c':  // cedille
+		switch (c.toLatin1()){
+		case 'c': return QChar(0xE7);
+		case 'C': return QChar(0xC7);
+		default: return c;
+		}
 	}
+	return c;
 }
 
 QString latexToPlainWord(const QString& word) {
+
 	/*	QString result=word;
  for (QList<QPair<QString,QString> >::const_iterator it=latexToPlainWordReplaceList.begin(); it!=latexToPlainWordReplaceList.end(); ++it)
   result.replace(it->first,it->second);*/
@@ -210,29 +283,41 @@ QString latexToPlainWord(const QString& word) {
 			//decode all meta characters starting with a backslash (c++ syntax: don't use an actual backslash there or it creates a multi line comment)
 			i++;
 			if (i>=word.length()) break;
-            switch (word[i].toLatin1()) {
+			switch (word[i].toLatin1()) {
 			case '-': //Trennung [separation] (german-babel-package also: \")
 			case '/': //ligatur preventing (german-package also: "|)
 				break;
 				
 			case '"':
+            case '\'':
+            case '^':
+            case '`':
+            case '~':
+            case 'c':
 				if (i+3 < word.length()) {
 					if (word[i+1] == '{' && word[i+3] == '}') {
-						result.append(transformCharacter(word[i+2]));
+						result.append(transformCharacter(word[i+2], word[i]));
 						i+=3;
-					} else if (word[i+1] == '\\' || word[i+1] == '"');  //ignore "
-					else i--; //repeat with "
-				} else if (i +1 < word.length() && (word[i+1] == '\\' || word[i+1] == '"'));  //ignore "
-				else i--; //repeat with "
+                        break;
+                    }
+                }
+                if (i +1 < word.length()){
+                    if(word[i+1] == '\\' || word[i+1] == '"')
+                        break;  //ignore "
+                    result.append(transformCharacter(word[i+1], word[i]));
+                    i++;
+                    break;
+                }
+                i--; //repeat with "
 				break;
 			default:
 				i--; //repeat with current char
 			}
-		} else if (word[i] == '"') {
+        } /* else if (word[i] == '"') {     // replacement from german package is handled extra
 			//decode all meta characters starting with "
 			i++;
 			if (i>=word.length()) break;
-            switch (word[i].toLatin1()) {
+			switch (word[i].toLatin1()) {
 			case '~':
 				result.append('-'); //- ohne Trennung (without separation)
 				break;
@@ -241,30 +326,38 @@ QString latexToPlainWord(const QString& word) {
 			case '"':  //ignore ""
 				break;
 			default:
-				result.append(transformCharacter(word[i]));
+                result.append(transformCharacter(word[i], '"'));
 				
 			}
-		} else result.append(word[i]);
+        }*/ else result.append(word[i]);
 	}
 	
 	return result;
 }
-
-QString extractSectionName(const QString& word, bool precut) {
-	int i=0;
-	int start=-1;
-	if(!precut) start=word.indexOf("{",i);
-	i=start>-1 ? start : 0;
-	int stop=word.indexOf("}",i);
-	i=word.indexOf("{",i+1);
-	while (i>0 && stop>0 && i<stop) {
-		stop=word.indexOf("}",stop+1);
-		i=word.indexOf("{",i+1);
-	}
-	if (stop<0) stop=word.length();
-	return word.mid(start+1,stop-start-1);
+QString latexToPlainWordwithReplacementList(const QString& word,QMap<QString,QString> &replacementList ) {
+    QString result;
+    QString w=latexToPlainWord(word);
+    if(replacementList.isEmpty())
+        return w;
+    while(!w.isEmpty()){
+        bool replaced=false;
+        foreach(const QString elem,replacementList.keys()){
+            if(w.startsWith(elem)){
+                result.append(replacementList.value(elem));
+                w=w.mid(elem.length());
+                replaced=true;
+                break;
+            }
+        }
+        if(!replaced){
+            result.append(w.left(1));
+            w=w.mid(1);
+        }
+    }
+    return result;
 }
 
+
 int findClosingBracket(const QString& word,int &start,QChar oc,QChar cc) {
 	int i=0;
 	if(start<0) start=word.indexOf(oc,i);
@@ -485,13 +578,14 @@ bool findTokenWithArg(const QString &line,const QString &token, QString &outName
 	return false;
 }
 
-bool findCommandWithArg(const QString &line,QString &cmd, QString &outName, QString &outArg, QString &remainder,int &optionStart){
+bool findCommandWithArg(const QString &line,QString &cmd, QString &outName, QString &outArg, QString &remainder,int &argStart,QString &option){
     // true means that a command is found, with or without arguments ...
     // otherwise a command before the interesting command leads to quiting the loop
 	outName="";
-	outArg="";
+	outArg = "";
 	remainder="";
-	optionStart=-1;
+    argStart=-1;
+    option="";
 	QRegExp token("\\\\\\w+\\*?");
 	int tagStart=token.indexIn(line);
 	int commentStart=line.indexOf(QRegExp("(^|[^\\\\])%")); // find start of comment (if any)
@@ -503,8 +597,9 @@ bool findCommandWithArg(const QString &line,QString &cmd, QString &outName, QStr
         if(values.size()>0){
             QString first=values.takeFirst();
             int start=starts.takeFirst();
-            if(first.startsWith('[')){
+            if(first.startsWith('[')){ // neglect [] arguments before {}
                 if(values.size()>0){
+                    option=LatexParser::removeOptionBrackets(first);
                     first=values.takeFirst();
                     start=starts.takeFirst();
                     if(first.startsWith('[')){
@@ -516,9 +611,16 @@ bool findCommandWithArg(const QString &line,QString &cmd, QString &outName, QStr
                     return true;
                 }
             }
-            optionStart=start+1;
+            argStart=start+1;
             remainder=line.mid(start+first.length());
-            outName=LatexParser::removeOptionBrackets(first);
+            outName = LatexParser::removeOptionBrackets(first);
+            if(values.size()>0){ // if there's something after the firt option (in case of import)
+                first=values.takeFirst();
+                start=starts.takeFirst();
+                argStart=start+1;
+                remainder=line.mid(start+first.length());
+                outArg = LatexParser::removeOptionBrackets(first);
+            }
             return true;
         }
         remainder=line.mid(tagStart+cmd.length());
@@ -652,6 +754,37 @@ QStringList regExpFindAllMatches(const QString& searchIn, const QRegExp& regexp,
 	return res;
 }
 
+/*
+ * a multi-match equivalent of QString::indexOf(QString)
+ */
+QList<int> indicesOf(const QString& line, const QString& word, Qt::CaseSensitivity cs) {
+	QList<int> columns;
+	int col = 0;
+	while (col < line.length()-1) {
+		col = line.indexOf(word, col, cs);
+		if (col < 0) break;
+		columns.append(col);
+		col++;
+	}
+	return columns;
+}
+
+/*
+ * a multi-match equivalent of QString::indexOf(QRegExp)
+ */
+QList<int> indicesOf(const QString& line, const QRegExp &rx) {
+	QList<int> columns;
+	int col = 0;
+	// exact match
+	while (col < line.length()-1) {
+		col = line.indexOf(rx, col);
+		if (col < 0) break;
+		columns.append(col);
+		col++;
+	}
+	return columns;
+}
+
 void addEnvironmentToDom(QDomDocument& doc,const QString& EnvironName,const QString& EnvironMode){
 	QDomElement root= doc.documentElement();
 	QDomElement tag = doc.createElement("context");
@@ -694,6 +827,35 @@ void addEnvironmentToDom(QDomDocument& doc,const QString& EnvironName,const QStr
 	root.insertBefore(tag,insertAt);
 }
 
+// adds entries for structure commands to the Dom of a QNFA file
+void addStructureCommandsToDom(QDomDocument &doc ,const QList<QStringList> &structureCommandLists) {
+	QDomElement root= doc.documentElement();
+
+	QDomNode parent;
+	for (int i=root.childNodes().size()-1; i>=0; i--) {
+		if (root.childNodes().item(i).attributes().namedItem("id").nodeValue() == "keywords/structure"){
+			parent = root.childNodes().item(i);
+			break;
+		}
+	}
+
+	for (int level=0; level<structureCommandLists.length(); level++) {
+		foreach (const QString &cmd, structureCommandLists[level]) {
+			QDomElement child = doc.createElement("word");
+			QString name = cmd;
+			name.remove('\\');
+			child.setAttribute("parenthesis",QString("%1:boundary@nomatch").arg(name));
+			child.setAttribute("parenthesisWeight", QString("%1").arg(8 - level));
+			child.setAttribute("fold","true");
+			name = cmd;
+			name.replace('\\', "\\\\");  // words are regexps, so we have to escape the slash
+			QDomText dtxt = doc.createTextNode(name);
+			child.appendChild(dtxt);
+			parent.appendChild(child);
+		}
+	}
+}
+
 void LatexParser::resolveCommandOptions(const QString &line, int column, QStringList &values, QList<int> *starts){
 	const QString BracketsOpen("[{");
 	const QString BracketsClose("]}");
@@ -764,6 +926,14 @@ QString LatexParser::removeOptionBrackets(const QString &option) {
 	return option;
 }
 
+int LatexParser::structureCommandLevel(const QString &cmd) const
+{
+	for (int i=0; i<structureCommandLists.length(); i++) {
+		if (structureCommandLists[i].contains(cmd)) return i;
+	}
+	return -1;
+}
+
 int LatexParser::findContext(QString &line,int &column) const{
 	if(line.isEmpty())
 		return 0;
@@ -846,20 +1016,21 @@ int LatexParser::findContext(QString &line,int &column) const{
 
 LatexParser::ContextType LatexParser::findContext(const QString &line, int column, QString &command, QString& value) const{
 	command=line;
+    int col=column; //remember column
 	int temp=findContext(command,column);
 	QStringList vals;
 	resolveCommandOptions(line,column,vals);
 	value="";
 	if(!vals.isEmpty()){
 		value=vals.takeFirst();
-		if(value.startsWith('[')){
+        if(value.startsWith('[') && temp<3){
 			if(!vals.isEmpty()){
 				value=vals.takeFirst();
 			}
 		}
-		if(value.startsWith('{'))
+        if(value.startsWith('{') || value.startsWith('['))
 			value.remove(0,1);
-		if(value.endsWith('}'))
+        if(value.endsWith('}')||value.endsWith(']'))
 			value.chop(1);
 	}
 	switch (temp) {
@@ -881,6 +1052,53 @@ LatexParser::ContextType LatexParser::findContext(const QString &line, int colum
         else if (possibleCommands["%graphics"].contains(command))
             return Graphics;
 		else return Option;
+    case 3:
+        // find possible commands for keyval completion
+        {
+            /*QString elem;
+            foreach(elem,possibleCommands.keys()){
+                if(elem.startsWith("key%") && (elem.mid(4)==command || elem.mid(4)==command+"#c"))
+                    break;
+                elem.clear();
+            }*/
+            QStringList keys=possibleCommands.keys();
+            QString arg;
+            if(!vals.isEmpty()){
+                arg=vals.first();
+                if(arg.startsWith('{') )
+                    arg.remove(0,1);
+                if(arg.endsWith('}'))
+                    arg.chop(1);
+            }
+            bool handled=false;
+            QString elem;
+            QStringList checkOptions;
+            checkOptions << "key%"+command+"/"+arg << "key%"+command+"/"+arg+"#c" << "key%"+command << "key%"+command+"#c";
+
+            foreach(elem,checkOptions){
+                if(keys.contains(elem)){
+                    handled=true;
+                    command=elem.mid(4);
+                    break;
+                }
+            }
+
+            if(handled){
+                // check that cursor is within keyval
+                bool isKey=false;
+                for(int i=col;col>0;col--){
+                    if(line.at(i-1).isLetter())
+                        continue;
+                    if(line.at(i-1)=='[' || line.at(i-1)==',')
+                        isKey=true;
+                    break;
+                }
+                if(isKey)
+                    return Keyval;
+                else
+                    return KeyvalValue;
+            }
+        }
 	default: return Unknown;
 	}
 }
@@ -993,6 +1211,19 @@ QString dequoteStr(const QString &s) {
 	return res;
 }
 
+/** add a quotation around the string if it does not already have one. **/
+QString quotePath(const QString &s) {
+	if (s.startsWith('"') || !s.contains(' ')) return QString(s);
+	return QString("\"%1\"").arg(s);
+}
+
+/** if the string is surrounded by qoutes, remove these **/
+QString removeQuote(const QString &s) {
+	if (s.length() >= 2 && s.startsWith('"') && s.endsWith('"')) {
+		return s.mid(1, s.length()-2);
+	}
+	return s;
+}
 
 
 QTextCodec* QTextCodecForTeXShopName(const QByteArray& enc){
@@ -1103,12 +1334,12 @@ QStringList LatexParser::latexNamesForTextCodec(const QTextCodec *codec) {
 		//case 2025 : return QStringList(); // "GB2312"
 		//case 2026 : return QStringList(); // "Big5"
 		//case 2084 : return QStringList(); // "KOI8-R"
-		//case 2086 : return QStringList(); // "IBM866"
+		case 2086 : return QStringList() << "cp866"; // "IBM866"
 		//case 2088 : return QStringList(); // "KOI8-U"
 		//case 2101 : return QStringList(); // "Big5-HKSCS"
 		//case 2107 : return QStringList(); // "TSCII"
 		case 2250 : return QStringList() << "cp1250"; // "windows-1250"
-		//case 2251 : return QStringList(); // "windows-1251"
+		case 2251 : return QStringList() << "cp1251"; // "windows-1251"
 		case 2252 : return QStringList() << "cp1252" << "ansinew"; // "windows-1252"
 		//case 2253 : return QStringList(); // "windows-1253"
 		//case 2254 : return QStringList(); // "windows-1254"
@@ -1134,16 +1365,6 @@ QStringList LatexParser::latexNamesForTextCodec(const QTextCodec *codec) {
 	return QStringList();
 }
 
-int lineEnd(const QByteArray& data, int index){
-	int r = data.indexOf('\n',index);
-	if (r != -1) return r;
-	r = data.indexOf('\r',index);
-	if (r != -1) return r;
-	r = data.indexOf("\x20\x29",index);
-	if (r != -1) return r;
-	return data.size();
-}
-
 QTextCodec * guessEncodingBasic(const QByteArray& data, int * outSure){
 	const char* str = data.data();
 	int size = data.size();
@@ -1185,7 +1406,7 @@ QTextCodec * guessEncodingBasic(const QByteArray& data, int * outSure){
 				if (badUtf8 == 0) sure = 0;
 			}
 		}
-	}
+	} else sure = 0;
 	if (outSure) *outSure = sure;
 	return guess;
 }
@@ -1228,49 +1449,68 @@ void LatexParser::guessEncoding(const QByteArray& data, QTextCodec *&guess, int
 		}
 		index = data.indexOf('=', index + 1);
 	}
-	
-	//search for \usepackage[.*]{inputenc} outside of a comment
-	index = data.indexOf("]{inputenc}");
-	while (index >= 0 && index < headerSize) {
-		int previous = data.lastIndexOf("\\usepackage[",index);
-		if (previous >= 0){
-			int commentStart = data.lastIndexOf('%',index);
-			int commentEnd = lineEnd(data, index);
-			if (commentStart <= commentEnd) {
-				QString encoding = QString(data.mid(previous+12, index - (previous + 12)));
-				QTextCodec* codec = QTextCodecForLatexName(encoding);
-				if (codec) {
-					sure = 100;
-					guess = codec;
-					return;
-				}
-			}
+
+	QString encoding = getEncodingFromPackage(data, headerSize, "inputenc");
+	if (encoding.isEmpty())
+		encoding = getEncodingFromPackage(data, headerSize, "inputenx");
+	if (!encoding.isEmpty()) {
+		QTextCodec* codec = QTextCodecForLatexName(encoding);
+		if (codec) {
+			sure = 100;
+			guess = codec;
+			return;
 		}
-		index = data.indexOf("]{inputenc}", index + 1);
-	}
-	
-	//search for \usepackage[.*]{inputenx} outside of a comment
-	index = data.indexOf("]{inputenx}");
-	while (index >= 0 && index < headerSize) {
-		int previous = data.lastIndexOf("\\usepackage[",index);
-		if (previous >= 0){
-			int commentStart = data.lastIndexOf('%',index);
-			int commentEnd = lineEnd(data,index);
-			if (commentStart <= commentEnd) {
-				QString encoding = QString(data.mid(previous+12, index - (previous + 12)));
-				QTextCodec* codec = QTextCodecForLatexName(encoding);
-				if (codec) {
-					sure = 100;
-					guess = codec;
-					return;
-				}
-			}
-		}
-		index = data.indexOf("]{inputenx}", index + 1);
 	}
 	return;
 }
 
+int LatexParser::lineStart(const QByteArray& data, int index) {
+	int n = qMax(data.lastIndexOf('\n', index), data.lastIndexOf('\r', index));
+	int o = data.lastIndexOf("\x20\x29",index);
+	if (n<0 && o<0) return 0;
+	if (n > o) return n+1; // skip over character
+	else return n+2; // skip over both chars
+}
+
+int LatexParser::lineEnd(const QByteArray& data, int index) {
+	int n = data.indexOf('\n',index);
+	int r = data.indexOf('\r',index);
+	if (n<0) n=r; // prevent non-existing value (-1) to be smaller than existing one in qMin
+	else if (r<0) r=n;
+	n = qMin(n, r);
+
+	r = data.indexOf("\x20\x29",index);
+	if (n<0) n=r;
+	else if (r<0) r=n;
+	n = qMin(n, r);
+	if (n >= 0) return r;
+	return data.size();
+}
+
+//search for first \usepackage[.*]{<packageName>} outside of a comment
+// returns the string inside the square brackets
+QString LatexParser::getEncodingFromPackage(const QByteArray& data, int headerSize, const QString &packageName) {
+	QByteArray packageEndToken(QString("]{%1}").arg(packageName).toLatin1());
+	QByteArray packageStartToken("\\usepackage[");
+	int index = data.indexOf(packageEndToken);
+	while (index >= 0 && index < headerSize) {
+		int lStart = lineStart(data, index);
+		int lEnd = lineEnd(data, index);
+		QByteArray line(data.mid(lStart, lEnd-lStart));
+		int encEnd = index - lStart;
+		int encStart = line.lastIndexOf(packageStartToken, encEnd);
+		if (encStart >= 0){
+			encStart += packageStartToken.length();
+			int commentStart = line.lastIndexOf('%', encEnd);
+			if (commentStart < 0) {
+				return QString(line.mid(encStart, encEnd - encStart));
+			}
+		}
+		index = data.indexOf(packageEndToken, index + 1);
+	}
+	return QString();
+}
+
 void LatexParser::append(const LatexParser& elem){
 	QHash<QString, QSet<QString> >::const_iterator i = elem.possibleCommands.constBegin();
 	while (i != elem.possibleCommands.constEnd()) {
@@ -1322,10 +1562,17 @@ void LatexParser::importCwlAliases(){
 	}
 }
 
-LatexReader::LatexReader():lp(&LatexParser::getInstance()){Q_ASSERT(this->lp);}
+LatexReader::LatexReader():index(0),wordStartIndex(0),lp(&LatexParser::getInstance()){Q_ASSERT(this->lp);}
 LatexReader::LatexReader(const QString& line): lp(&LatexParser::getInstance()){Q_ASSERT(this->lp);setLine(line);}
 LatexReader::LatexReader(const LatexParser& lp, const QString& line):lp(&lp){Q_ASSERT(this->lp);setLine(line);}
 
+LatexReader::LatexReader(const LatexParser &lp, const QString &line, QMap<QString, QString> &replacementList):lp(&lp)
+{
+    Q_ASSERT(this->lp);
+    setLine(line);
+    mReplacementList=replacementList;
+}
+
 
 LatexReader::NextWordFlag LatexReader::nextWord(bool returnCommands){
 	int reference=-1;
@@ -1419,7 +1666,8 @@ LatexReader::NextWordFlag LatexReader::nextWord(bool returnCommands){
 			//if (word.length() == 2 && word[0] == '"' && CommonEOW.contains(word[1]))
 			//	return NW_PUNCTATION; //some quotation mark
 			if (word.length() > 1 && (word.contains('\\')||word.contains('"'))){
-				word=latexToPlainWord(word); //remove special chars			
+                //word=latexToPlainWord(word); //remove special chars
+                word=latexToPlainWordwithReplacementList(word,mReplacementList); //remove special chars
 				if (word.isEmpty()) continue;
 			}
 			if (lp->environmentCommands.contains(lastCommand))
@@ -1469,11 +1717,13 @@ LatexPackage loadCwlFile(const QString fileName,LatexCompleterConfig *config) {
 	if (!fn.isEmpty() && tagsfile.open(QFile::ReadOnly)) {
 	    QString line;
 	    QTextStream stream(&tagsfile);
+		stream.setCodec("UTF-8");
 	    QRegExp rxCom("^(\\\\\\w+\\*?)(\\[.+\\])*\\{(.*)\\}");
 	    QRegExp rxCom2("^(\\\\\\w+\\*?)\\[(.+)\\]");
 	    rxCom.setMinimal(true);
 	    QStringList keywords;
 	    keywords << "text" << "title";
+        QString keyvals;
 	    while (!stream.atEnd()) {
 		line = stream.readLine().trimmed();
 		if(line.startsWith("#include:")){
@@ -1484,6 +1734,27 @@ LatexPackage loadCwlFile(const QString fileName,LatexCompleterConfig *config) {
 			    package.requiredPackages<<fn+".cwl";
 		    }
 		}
+        if(line.startsWith("#keyvals:")){
+            // start reading keyvals
+            keyvals=line.mid(9);
+            continue;
+        }
+        if(line.startsWith("#endkeyvals")){
+            // end of reading keyvals
+            keyvals.clear();
+            continue;
+        }
+        if(!keyvals.isEmpty()){
+            // read keyval (name stored in "keyvals")
+            package.possibleCommands["key%"+keyvals] << line;
+            continue;
+        }
+        if(line.startsWith("#repl:")){
+            // start reading keyvals
+            package.possibleCommands["%replace"] << line.mid(6);
+            continue;
+        }
+
 		if (!line.isEmpty() && !line.startsWith("#") && !line.startsWith(" ")) {
 		    //hints for commands usage (e.g. in mathmode only) are separated by #
 		    int sep=line.indexOf('#');
@@ -1492,26 +1763,26 @@ LatexPackage loadCwlFile(const QString fileName,LatexCompleterConfig *config) {
 		    bool uncommon=false;
 		    bool hideFromCompletion=false;
 		    if(sep>-1){
-			valid=line.mid(sep+1);
-			line=line.left(sep);
-			if(valid.startsWith("*")){
-			    valid=valid.mid(1);
-			    uncommon=true;
-			}
-			if(valid.startsWith("/")){
-			    env=valid.mid(1).split(',');
-			    valid="e";
-			}
-			if(valid.contains("\\")){
-			    int i=valid.indexOf("\\");
-			    QString zw=valid.mid(i+1);
-			    env=zw.split(',');
-			    valid=valid.left(i);
-			}
-			if(valid.contains('S')){
-			    hideFromCompletion=true;
-			    valid.remove('S');
-			}
+				valid=line.mid(sep+1);
+				line=line.left(sep);
+				if(valid.startsWith("*")){
+					valid=valid.mid(1);
+					uncommon=true;
+				}
+				if(valid.startsWith("/")){
+					env=valid.mid(1).split(',');
+					valid="e";
+				}
+				if(valid.contains("\\")){
+					int i=valid.indexOf("\\");
+					QString zw=valid.mid(i+1);
+					env=zw.split(',');
+					valid=valid.left(i);
+				}
+				if(valid.contains('S')){
+					hideFromCompletion=true;
+					valid.remove('S');
+				}
 		    }
             // parse for spell checkable commands
             int res=rxCom.indexIn(line);
@@ -1614,6 +1885,12 @@ LatexPackage loadCwlFile(const QString fileName,LatexCompleterConfig *config) {
                 }
                 valid.remove('U');
             }
+			if(valid.contains('D')){ // todo command
+				if(res>-1){
+					package.possibleCommands["%todo"] << rxCom.cap(1);
+				}
+				valid.remove('D');
+			}
 		    // normal commands for syntax checking
 		    // will be extended to distinguish between normal and math commands
 		    if(valid.isEmpty() || valid.contains('n')){
@@ -1657,24 +1934,30 @@ LatexPackage loadCwlFile(const QString fileName,LatexCompleterConfig *config) {
 			    package.possibleCommands["tabbing"] << line.simplified();
 			}
 		    }
-		    if(valid.contains('e') && !env.isEmpty()){ // tabbing support
-			if(res==-1){
-			    foreach(const QString& elem,env)
-				package.possibleCommands[elem] << line.simplified();
-			}
-		    }
+            if(valid.contains('e') && !env.isEmpty()){ // tabbing support
+                if(res==-1){
+                    foreach(const QString& elem,env)
+                        package.possibleCommands[elem] << line.simplified();
+                }else{
+                    QString cmd=rxCom.cap(1);
+                    if(cmd=="\\begin" || cmd=="\\end"){
+                        cmd=line.simplified();
+                    }
+                    foreach(const QString& elem,env)
+                        package.possibleCommands[elem] << cmd;
+                }
+            }
 		    if(!valid.contains('e') && !env.isEmpty()){ // set env alias
-			if(res>-1){
-			    if(rxCom.cap(1)=="\\begin"){
-				QString envName=rxCom.cap(3);
-				if(!envName.isEmpty()){
-
-				    foreach(const QString& elem,env)
-					package.environmentAliases.insert(rxCom.cap(3),elem);
-				}
-			    }
-			}
-		    }
+                if(res>-1){
+                    if(rxCom.cap(1)=="\\begin"){
+                        QString envName=rxCom.cap(3);
+                        if(!envName.isEmpty()){
+                            foreach(const QString& elem,env)
+                                package.environmentAliases.insert(rxCom.cap(3),elem);
+                        }
+                    }
+                }
+            }
 		    // normal parsing for completer
 		    if(hideFromCompletion)
 			continue; // command for spell checking only (auto parser)
@@ -1788,11 +2071,11 @@ void showTooltipLimited(QPoint tt,QString topic,int width){
     lLabel.setText(topic);
     lLabel.adjustSize();
 
-
     int textWidthInPixels = lLabel.width()+10; // +10 good guess
 
-    if (screen.width()-textWidthInPixels>=tt.x()) QToolTip::showText(tt, topic);//-90
-    else {
+    if (screen.width()-textWidthInPixels>=tt.x()) {
+        QToolTip::showText(tt, topic);//-90
+    } else {
         //list->mapToGlobal
         QPoint tt2(tt.x()-textWidthInPixels-width, tt.y());
 
@@ -1817,8 +2100,10 @@ void showTooltipLimited(QPoint tt,QString topic,int width){
                     maxLine=elem;
                 }
             }
+            int averageWidth=lLabel.fontMetrics().averageCharWidth();
+            maxLength=qMin(maxLength,w/averageWidth);
             while(textWidthInPixels>w && maxLength>10){
-                maxLength--;
+                maxLength-=2;
                 lLabel.setText(maxLine.left(maxLength));
                 lLabel.adjustSize();
                 textWidthInPixels=lLabel.width()+10;
@@ -1834,3 +2119,31 @@ void showTooltipLimited(QPoint tt,QString topic,int width){
         QToolTip::showText(tt2, topic);
     }
 }
+
+QString truncateLines(const QString & s, int maxLines) {
+	int lineCount = 0;
+	for (int i=0; i<s.length(); i++) {
+		if (s[i] == '\n') lineCount++;
+		if (lineCount >= maxLines) {
+			return s.left(i+1) + "...";
+		}
+	}
+	return s;
+}
+
+/*
+ * Utility function for most recent strings, e.g. for filenames
+ * The item is inserted at the front and removed if present in the rest of the list.
+ * The list will not get longer than maxLength.
+ * Returns true if the list contents changed (i.e. item was not already in first place)
+ */
+bool addMostRecent(const QString & item, QStringList & mostRecentList, int maxLength) {
+	int p = mostRecentList.indexOf(item);
+	bool changed = (p != 0);
+	if (!changed) return changed;
+
+	if (p > 0) mostRecentList.removeAt(p);
+	mostRecentList.prepend(item);
+	if (mostRecentList.count() > maxLength) mostRecentList.removeLast();
+	return changed;
+}
