@@ -40,8 +40,7 @@ QString getCommonEOW();
 
 //removes special latex characters
 QString latexToPlainWord(const QString& word);
-//extracts the section name after \section is removed (brackets removal)
-QString extractSectionName(const QString& word,bool precut=false);
+QString latexToPlainWordwithReplacementList(const QString& word, QMap<QString, QString> &replacementList );
 //closing bracket (opening and closing bracket considered correctly)
 int findClosingBracket(const QString& word,int &start,QChar oc=QChar('{'),QChar cc=QChar('}'));
 //opening bracket (opening and closing bracket considered correctly), start at "start"
@@ -73,16 +72,19 @@ QString findToken(const QString &line,const QString &token,int &start);
 QString findToken(const QString &line,QRegExp &token);
 // find token (e.g. \label \input \section and return content (\newcommand{name}[arg]), returns true if outName!=""
 bool findTokenWithArg(const QString &line,const QString &token, QString &outName, QString &outArg);
-bool findCommandWithArg(const QString &line,QString &cmd, QString &outName, QString &outArg, QString &remainder,int &optionStart);
+bool findCommandWithArg(const QString &line, QString &cmd, QString &outName, QString &outArg, QString &remainder, int &argStart, QString &option);
 
 
 // generate multiple times used regexpression
 QRegExp generateRegExp(const QString &text,const bool isCase,const bool isWord, const bool isRegExp);
 
 QStringList regExpFindAllMatches(const QString& searchIn, const QRegExp& regexp, int cap = 0);
+QList<int> indicesOf(const QString& line, const QString& word, Qt::CaseSensitivity cs = Qt::CaseSensitive);
+QList<int> indicesOf(const QString& line, const QRegExp& rx);
 
 // add Environment to QNFA DOM
 void addEnvironmentToDom(QDomDocument &doc,const QString& EnvironName,const QString& EnvironMode);
+void addStructureCommandsToDom(QDomDocument &doc ,const QList<QStringList> &structureCommandLists);
 
 QString intListToStr(const QList<int> &ints);
 QList<int> strToIntList(const QString &s);
@@ -93,6 +95,9 @@ QString formatJsonStringParam(const QString &id, const QString &val, int minIdWi
 QString enquoteStr(const QString &s);
 QString dequoteStr(const QString &s);
 
+QString quotePath(const QString &s);
+QString removeQuote(const QString &s);
+
 QTextCodec * guessEncodingBasic(const QByteArray& data, int * outSure);
 
 enum {
@@ -106,13 +111,17 @@ enum {
 
 QString getImageAsText(const QPixmap &AImage);
 void showTooltipLimited(QPoint tt,QString topic,int width=0);
+QString truncateLines(const QString & s, int maxLines);
+
+bool addMostRecent(const QString & item, QStringList & mostRecentList, int maxLength);
 
 class LatexParser{
+	friend class SmallUsefulFunctionsTest;
 public:
 	LatexParser();
 	void init();
 
-    enum ContextType {Unknown, Command, Environment, Label, Reference, Citation, Citation_Ext, Option, Graphics,Package};
+    enum ContextType {Unknown, Command, Environment, Label, Reference, Citation, Citation_Ext, Option, Graphics,Package,Keyval,KeyvalValue};
 	// realizes whether col is in a \command or in a parameter {}
 	int findContext(QString &line, int &column) const;
 	
@@ -141,6 +150,9 @@ public:
 	QSet<QString> customCommands;
     //QSet<QString> graphicsIncludeCommands;
 	QStringList structureCommands;
+	QList<QStringList> structureCommandLists;  // a list for each level. 0:\part,\mypart 1:\chapter,\mychapter 2:\section ... 5:paragraph
+	int structureDepth() { return structureCommandLists.length(); }
+	int structureCommandLevel(const QString &cmd) const;
 	QMultiHash<QString,QString> packageAliases; // aliases for classes to packages e.g. article = latex-document, latex-mathsymbols, etc
 	QMultiHash<QString,QString> environmentAliases; // aliases for environments, e.g. equation is math, supertabular is also tab etc.
 	// commands used for syntax check (per doc basis)
@@ -155,6 +167,10 @@ public:
 	static void guessEncoding(const QByteArray& data, QTextCodec *&guess, int &sure);
 	
 	static LatexParser& getInstance();
+private:
+	static int lineStart(const QByteArray& data, int index);
+	static int lineEnd(const QByteArray& data, int index);
+	static QString getEncodingFromPackage(const QByteArray& data, int headerSize, const QString &packageName);
 };
 
 Q_DECLARE_METATYPE(LatexParser)
@@ -163,9 +179,10 @@ struct LatexReader{
 	LatexReader();
 	LatexReader(const QString& line);
 	LatexReader(const LatexParser& lp, const QString& line);
+    LatexReader(const LatexParser& lp, const QString& line, QMap<QString,QString> &replacementList);
 	
 	/** searches the next token in the line line after/at the index index
-	//there are these possible kind of tokens % (which starts a comment), { or } (as parantheses), \.* (command) or .* (text)
+	//there are these possible kind of tokens % (which starts a comment), { or } (as parentheses), \.* (command) or .* (text)
 	 \param index returns the index of the first character after the word
 	 \param inOption Don't stop at eow characters
 	 \param detectMath If true, returns $ $$ _ ^ & as commands
@@ -209,8 +226,10 @@ struct LatexReader{
 	QString word;
 	QString lastCommand;
 	int wordStartIndex;
+
 private:
 	const LatexParser* lp;
+    QMap<QString, QString> mReplacementList;
 };
 
 
