@@ -46,6 +46,7 @@ private:
 
 Q_DECLARE_METATYPE(TestToken);
 Q_DECLARE_METATYPE(QList<TestToken>);
+Q_DECLARE_METATYPE(QList<int>);
 
 /*QList<TestToken>& operator<< (QList<TestToken> & list, const char* str){
 	return list << TestToken(str);
@@ -110,12 +111,12 @@ class SmallUsefulFunctionsTest: public QObject{
 		QTest::addColumn<QString >("str");
 		QTest::addColumn<QList<TestToken> >("tokens");
 
-		addRow("simple whitespace", filter,
+        /*addRow("simple whitespace", filter,
 			QList<TestToken>() << "abcde" << "    " << "fghik" << "\t" << "Mice");
 		addRow("simple eow", filter,
-		       QList<TestToken>() << "abcde" << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) << "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION));
+               QList<TestToken>() << "abcde" << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) );//<< "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION)); "-" is now added to the word ...
 		addRow("simple eow", filter,
-		       QList<TestToken>() << "abcde." << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) << "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION));
+               QList<TestToken>() << "abcde." << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) );//<< "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION));
 		addRow("environment+comment",filter,
 			QList<TestToken>() << "Test1234" << "\\begin" << "{" << env("environment") << "}" << "{" << "add" << "}" << "XYZ" << "!" << "!" << "!" << "\\command" << "%"     << "comment" << "\\COMMENT");
 		addRow("some environments", filter,
@@ -128,16 +129,23 @@ class SmallUsefulFunctionsTest: public QObject{
 		       QList<TestToken>() << "\\includegraphics" << "[" << option("ab") << TestToken(":", NW_OPTION_PUNCTATION) << "\\linewidth" << "]" << "{" << "\\abc" << " " << option("dfdf") << "\\xyz" << "}" << "continue");
 		addRow("comments", filter, QList<TestToken>() << "hallo" << " " << "welt" <<  "  " << "\\\\" << "normaltext" <<  "  " << TestToken("\\%",NW_IGNORED_TOKEN) << "!" << "!" << "!" << "stillNoComment" << "\\\\" << TestToken("\\%","%",NW_IGNORED_TOKEN) <<"  "<< "none" << "\\\\" << "%" << "comment" << "   " << "more" << " " << "comment");
 		addRow("escaped characters", filter, QList<TestToken>() << "hallo" << TestToken("\\%",NW_IGNORED_TOKEN) << "abc");
-		addRow("escaped characters", filter, QList<TestToken>() << "1234" << TestToken("\\%\\&\\_",NW_IGNORED_TOKEN)  << "567890");
+        addRow("escaped characters", filter, QList<TestToken>() << "1234" << TestToken("\\%\\&\\_",NW_IGNORED_TOKEN)  << "567890");  */
 		addRow("special characters", filter,
-               QList<TestToken>() << QString::fromUtf8("lösbar") << " " << TestToken("l\"osbar",QString::fromUtf8("lösbar"),LatexReader::NW_TEXT) << " " << TestToken("l\\\"osbar",QString::fromUtf8("lösbar"),LatexReader::NW_TEXT) << " " << TestToken("l\\\"{o}sbar",QString::fromUtf8("lösbar"),LatexReader::NW_TEXT) << " " << QString::fromUtf8("örtlich") <<" " <<TestToken("\"ortlich",QString::fromUtf8("örtlich"),LatexReader::NW_TEXT)<<" " <<TestToken("\\\"ortlich",QString::fromUtf8("örtlich"),LatexReader::NW_TEXT)<<" " <<TestToken("\\\"{o}rtlich",QString::fromUtf8("örtlich"),LatexReader::NW_TEXT) );
+               QList<TestToken>() << "lösbar" << " " << TestToken("l\"osbar",QString::fromUtf8("lösbar"),LatexReader::NW_TEXT) << " " << TestToken("l\\\"osbar",QString::fromUtf8("lösbar"),LatexReader::NW_TEXT) << " " << TestToken("l\\\"{o}sbar",QString::fromUtf8("lösbar"),LatexReader::NW_TEXT) << " " << "örtlich" <<" " <<TestToken("\"ortlich",QString::fromUtf8("örtlich"),LatexReader::NW_TEXT)<<" " <<TestToken("\\\"ortlich",QString::fromUtf8("örtlich"),LatexReader::NW_TEXT)<<" " <<TestToken("\\\"{o}rtlich",QString::fromUtf8("örtlich"),LatexReader::NW_TEXT) );
 	}
 	void nextWord_complex_test(bool commands){
 		//get data
 		QFETCH(QString, str);	
 		QFETCH(QList<TestToken>, tokens);	
 		int pos=0; int type;
-		LatexReader lr(str);
+        QMap<QString,QString> replacementList;
+        replacementList.insert("\"o",QString::fromUtf8("ö"));
+        replacementList.insert("\"a",QString::fromUtf8("ä"));
+        replacementList.insert("\"-","");
+        replacementList.insert("\"|","");
+        replacementList.insert("\"\"","");
+        LatexParser &lp=LatexParser::getInstance();
+        LatexReader lr(lp,str,replacementList);
 		while ((type=lr.nextWord(commands))!=LatexReader::NW_NOTHING) {
 			const int& startIndex = lr.wordStartIndex;
 			const QString& token = lr.word;
@@ -189,7 +197,14 @@ private slots:
 		QFETCH(QList<TestToken>, tokens);	
 		
 		int pos=0;
-		LatexReader lr(str);
+        QMap<QString,QString> replacementList;
+        replacementList.insert("\"o",QString::fromUtf8("ö"));
+        replacementList.insert("\"a",QString::fromUtf8("ä"));
+        replacementList.insert("\"-","");
+        replacementList.insert("\"|","");
+        replacementList.insert("\"\"","");
+        LatexParser &lp=LatexParser::getInstance();
+        LatexReader lr(lp,str,replacementList);
 		while (lr.nextTextWord()) {
 			const int& startIndex = lr.wordStartIndex;
 			const QString& token = lr.word;
@@ -251,8 +266,8 @@ private slots:
 		QTest::newRow("sepchar6")  << "?oz\\\"{a}di?" << 1 << false << false << (int)LatexReader::NW_TEXT << 10 << QString("oz%1di").arg(QChar(0xE4)) << 1;
 		QTest::newRow("sepchar7")  << "?oz\\\"adi?" << 1 << false << false << (int)LatexReader::NW_TEXT << 8 << QString("oz%1di").arg(QChar(0xE4)) << 1;
 		QTest::newRow("sepchar8")  << "?oz\"\"adi?" << 1 << false << false << (int)LatexReader::NW_TEXT << 8 << "ozadi" << 1;
-		QTest::newRow("sepchar8")  << "?oz\"yxdi?" << 1 << false << false << (int)LatexReader::NW_TEXT << 8 << "ozyxdi" << 1;
-		QTest::newRow("sepchar8")  << "?oz\"y?" << 1 << false << false << (int)LatexReader::NW_TEXT << 5 << "ozy" << 1;
+        //QTest::newRow("sepchar8")  << "?oz\"yxdi?" << 1 << false << false << (int)LatexReader::NW_TEXT << 8 << "ozyxdi" << 1;  //invalid combinations in german.sty
+        //QTest::newRow("sepchar8")  << "?oz\"y?" << 1 << false << false << (int)LatexReader::NW_TEXT << 5 << "ozy" << 1; //invalid combinations in german.sty
 		QTest::newRow("word end")  << "?no\"<di?" << 1 << false << false << (int)LatexReader::NW_TEXT << 3 << "no" << 1;
 		QTest::newRow("word end")  << "?yi''di?" << 1 << false << false << (int)LatexReader::NW_TEXT << 3 << "yi" << 1;
 		QTest::newRow("umlauts")  << "\"a\"o\"u\"A\"O\"U\\\"{a}\\\"{o}\\\"{u}\\\"{A}\\\"{O}\\\"{U}" << 0 << false << false << (int)LatexReader::NW_TEXT << 42 << (QString(QChar(0xE4))+QString(QChar(0xF6))+QString(QChar(0xFC))+QString(QChar(0xC4))+QString(QChar(0xD6))+QString(QChar(0xDC))+QString(QChar(0xE4))+QString(QChar(0xF6))+QString(QChar(0xFC))+QString(QChar(0xC4))+QString(QChar(0xD6))+QString(QChar(0xDC))) << 0; //unicode to be independent from c++ character encoding
@@ -267,7 +282,19 @@ private slots:
 		QFETCH(QString, outWord);
 		QFETCH(int, wordStartIndex);
 		
-		LatexReader lr(line);
+        //LatexReader lr(line);
+        QMap<QString,QString> replacementList;
+        replacementList.insert("\"o",QString::fromUtf8("ö"));
+        replacementList.insert("\"a",QString::fromUtf8("ä"));
+        replacementList.insert("\"u",QString::fromUtf8("ü"));
+        replacementList.insert("\"A",QString::fromUtf8("Ä"));
+        replacementList.insert("\"O",QString::fromUtf8("Ö"));
+        replacementList.insert("\"U",QString::fromUtf8("Ü"));
+        replacementList.insert("\"-","");
+        replacementList.insert("\"|","");
+        replacementList.insert("\"\"","");
+        LatexParser &lp=LatexParser::getInstance();
+        LatexReader lr(lp,line,replacementList);
 		lr.index = inIndex;
 		int rs=(int)(lr.nextWord(commands));
 
@@ -327,9 +354,9 @@ private slots:
 		QTest::newRow("label") << "\\label{test}" << 8 << (int)LatexParser::Label << "\\label" <<"test";
 		QTest::newRow("cite") << "\\cite{test}" << 8 << (int)LatexParser::Citation << "\\cite" <<"test";
 		QTest::newRow("cite") << "\\cite{test}" << 3 << (int)LatexParser::Command << "\\cite" <<"test";
-                QTest::newRow("abcd option") << "\\abcd{test}" << 7 << (int)LatexParser::Option << "\\abcd" <<"test";
-                QTest::newRow("abcd option2") << "\\abcd[abc]{test}" << 12 << (int)LatexParser::Option << "\\abcd" <<"test";
-                QTest::newRow("abcd option3") << "\\abcd[\\abc]{test}" << 12 << (int)LatexParser::Option << "\\abcd" <<"test";
+        QTest::newRow("abcd option") << "\\abcd{test}" << 7 << (int)LatexParser::Option << "\\abcd" <<"test";
+        QTest::newRow("abcd option2") << "\\abcd[abc]{test}" << 12 << (int)LatexParser::Option << "\\abcd" <<"test";
+        QTest::newRow("abcd option3") << "\\abcd[\\abc]{test}" << 12 << (int)LatexParser::Option << "\\abcd" <<"test";
 	}
 	void test_findContext2(){
 		QFETCH(QString, in);
@@ -338,12 +365,117 @@ private slots:
 		QFETCH(QString, command);
 		QFETCH(QString, value);
 		QString cmd;
-		QString val;
+        QString val;
 		LatexParser::ContextType res=LatexParser::getInstance().findContext(in,pos,cmd,val);
 		QEQUAL((int)res,out);
 		QEQUAL(cmd,command);
 		QEQUAL(val,value);
 	}
+	void test_resolveCommandOptions_data(){
+		QTest::addColumn<QString>("line");
+		QTest::addColumn<int>("column");
+		QTest::addColumn<QStringList>("expectedValues");
+		QTest::addColumn<QList<int> >("expectedStarts");
+
+		QTest::newRow("noOption") << "\\begin nothing" << 0 << (QStringList()) << (QList<int>());
+		QTest::newRow("singleOption") << "\\begin{test}" << 0 << (QStringList() << "{test}") << (QList<int>() << 6);
+		QTest::newRow("singleOptionSpace") << "\\begin {test}" << 0 << (QStringList() << "{test}") << (QList<int>() << 7);
+		QTest::newRow("multiOption") << "\\begin{test 1}{test 2}" << 0 << (QStringList() << "{test 1}" << "{test 2}") << (QList<int>() << 6 << 14);
+		QTest::newRow("multiOption2") << "\\begin[test 1]{test 2}" << 0 << (QStringList() << "[test 1]" << "{test 2}") << (QList<int>() << 6 << 14);
+		QTest::newRow("multiOption3") << "\\begin{test 1}[test 2]{test 3}" << 0 << (QStringList() << "{test 1}" << "[test 2]" << "{test 3}") << (QList<int>() << 6 << 14 << 22);
+		QTest::newRow("cmdStart") << "\\section{\\LaTeX rules}" << 0 << (QStringList() << "{\\LaTeX rules}") << (QList<int>() << 8);
+		QTest::newRow("cmdMid") << "\\section{a \\textit{nested} command}" << 0 << (QStringList() << "{a \\textit{nested} command}") << (QList<int>() << 8);
+	}
+	void test_resolveCommandOptions(){
+		QFETCH(QString, line);
+		QFETCH(int, column);
+		QFETCH(QStringList, expectedValues);
+		QFETCH(QList<int>, expectedStarts);
+		QStringList values;
+		QList<int> starts;
+		LatexParser::resolveCommandOptions(line, column, values, &starts);
+		QEQUAL(values.length(), starts.length());
+		QEQUAL(values.join("|"), expectedValues.join("|"));
+		QVERIFY(starts == expectedStarts);
+	}
+	void test_lineStart_data(){
+		QTest::addColumn<QString>("text");
+		QTest::addColumn<int>("pos");
+		QTest::addColumn<int>("start");
+
+		QTest::newRow("empty") << "" << 0 << 0;
+		QTest::newRow("start") << "foo bar" << 0 << 0;
+		QTest::newRow("text") << "foo bar" << 6 << 0;
+		QTest::newRow("newline") << "foo\nbar" << 6 << 4;
+		QTest::newRow("cr") << "foo\rbar" << 6 << 4;
+		QTest::newRow("cr_newline") << "foo\r\nbar" << 6 << 5;
+		QTest::newRow("start_newline") << "foo\nbar" << 4 << 4;
+	}
+	void test_lineStart(){
+		QFETCH(QString, text);
+		QFETCH(int, pos);
+		QFETCH(int, start);
+		QEQUAL(LatexParser::lineStart(text.toLatin1(), pos), start);
+	}
+	void test_lineEnd_data(){
+		QTest::addColumn<QString>("text");
+		QTest::addColumn<int>("pos");
+		QTest::addColumn<int>("end");
+
+		QTest::newRow("empty") << "" << 0 << 0;
+		QTest::newRow("end") << "foo bar" << 7 << 7;
+		QTest::newRow("text") << "foo bar" << 1 << 7;
+		QTest::newRow("newline") << "foo\nbar" << 1 << 3;
+		QTest::newRow("cr") << "foo\rbar" << 1 << 3;
+		QTest::newRow("cr_newline") << "foo\r\nbar" << 1 << 3;
+		QTest::newRow("end_newline") << "foo\nbar" << 3 << 3;
+	}
+	void test_lineEnd(){
+		QFETCH(QString, text);
+		QFETCH(int, pos);
+		QFETCH(int, end);
+		QEQUAL(LatexParser::lineEnd(text.toLatin1(), pos), end);
+	}
+	void test_getEncodingFromPackage_data() {
+		QTest::addColumn<QString>("text");
+		QTest::addColumn<QString>("encodingName");
+
+		QTest::newRow("empty") << "" << "";
+		QTest::newRow("no info") << "foo bar\nbaz" << "";
+		QTest::newRow("simple") << "\\usepackage[latin1]{inputenc}" << "latin1";
+		QTest::newRow("simple2") << "foo bar\nbaz\n\\usepackage[latin1]{inputenc}" << "latin1";
+		QTest::newRow("spaces") << "  \\usepackage[utf8]{inputenc}  " << "utf8";
+		QTest::newRow("multiple") << "\\usepackage[latin1]{inputenc}\n\\usepackage[utf8]{inputenc}" << "latin1";
+		QTest::newRow("commented") << "  %\\usepackage[latin1]{inputenc}" << "";
+		QTest::newRow("commented2") << "% \\usepackage[latin1]{inputenc}\n\\usepackage[utf8]{inputenc}" << "utf8";
+	}
+	void test_getEncodingFromPackage() {
+		QFETCH(QString, text);
+		QFETCH(QString, encodingName);
+		QEQUAL(encodingName, LatexParser::getEncodingFromPackage(text.toLatin1(), text.length(), "inputenc"));
+	}
+	void test_guessEncoding_data() {
+		QTest::addColumn<QString>("text");
+		QTest::addColumn<QString>("encodingName");
+		QTest::newRow("empty") << "" << "";
+		QTest::newRow("no info") << "foo bar\nbaz" << "";
+		QTest::newRow("inputenc") << "\\usepackage[latin1]{inputenc}" << "ISO-8859-1";
+		QTest::newRow("inputenx") << "\\usepackage[utf8]{inputenx}" << "UTF-8";
+	}
+	void test_guessEncoding() {
+		QFETCH(QString, text);
+		QFETCH(QString, encodingName);
+
+		QTextCodec *encoding = 0;
+		int sure = 0;
+		LatexParser::guessEncoding(text.toLatin1(), encoding, sure);
+		if (encodingName.isEmpty()) {
+            int b = (encoding==0) ? 0 : 1 ;
+            QEQUAL(0, b);
+		} else {
+			QEQUAL(encodingName, QString(encoding->name()));
+		}
+	}
 	void test_findClosingBracket_data(){
 		QTest::addColumn<QString>("line");
 		QTest::addColumn<int>("start");
@@ -481,6 +613,7 @@ private slots:
 			QEQUAL2(splittedLines[i], out[i], QString("in split #%1").arg(i));
 		}
 	}
+    /*
 	void test_getSimplifiedSVNVersion_data(){
 		QTest::addColumn<QString>("versionString");
 		QTest::addColumn<int>("out");
@@ -494,7 +627,7 @@ private slots:
 		QFETCH(QString, versionString);
 		QFETCH(int, out);
 		QEQUAL(getSimplifiedSVNVersion(versionString), out);
-	}
+    }*/
 	void test_minimalJsonParse_data(){
 		QTest::addColumn<QString>("jsonData");
 		QTest::addColumn<bool>("retVal");
@@ -613,6 +746,80 @@ private slots:
 		QEQUAL(getParamItem(line, pos, stopAtWhiteSpace), result);
 	}
 
+	void test_addMostRecent_data() {
+		QTest::addColumn<QString>("item");
+		QTest::addColumn<QStringList>("list");
+		QTest::addColumn<int>("maxLength");
+		QTest::addColumn<QStringList>("result");
+
+		QTest::newRow("addEmpty") << "item1" << QStringList() << 2 << (QStringList() << "item1");
+		QTest::newRow("addMax") << "item2" << (QStringList() << "item1") << 2 << (QStringList() << "item2" << "item1");
+		QTest::newRow("addBeyondMax") << "item3" << (QStringList() << "item1" << "item2") << 2 << (QStringList() << "item3" << "item1");
+		QTest::newRow("addExisting1") << "item1" << (QStringList() << "item1" << "item2") << 2 << (QStringList() << "item1" << "item2");
+		QTest::newRow("addExisting2") << "item2" << (QStringList() << "item1" << "item2") << 2 << (QStringList() << "item2" << "item1");
+	}
+
+	void test_addMostRecent() {
+		QFETCH(QString, item);
+		QFETCH(QStringList, list);
+		QFETCH(int, maxLength);
+		QFETCH(QStringList, result);
+
+		addMostRecent(item, list, maxLength);
+		QEQUAL(list.length(), result.length());
+		for (int i=0; i<list.length(); i++) {
+			QEQUAL(list[i], result[i]);
+		}
+	}
+
+	void test_indicesOf_data() {
+		QTest::addColumn<QString>("line");
+		QTest::addColumn<QString>("word");
+		QTest::addColumn<bool>("caseSensitive");
+		QTest::addColumn<QList<int> >("result");
+
+		QTest::newRow("noMatch") << "A test, a test, a test" << "foo" << false << (QList<int>());
+		QTest::newRow("multiCS") << "A test, a test, a test" << "a" << true << (QList<int>() << 8 << 16);
+		QTest::newRow("multiCSI") << "A test, a test, a test" << "a" << false << (QList<int>() << 0 << 8 << 16);
+		QTest::newRow("multiCS_end") << "A test, a test, a test" << "test" << true << (QList<int>() << 2 << 10 << 18);
+	}
+
+	void test_indicesOf() {
+		QFETCH(QString, line);
+		QFETCH(QString, word);
+		QFETCH(bool, caseSensitive);
+		QFETCH(QList<int>, result);
+
+		QList<int> indices = indicesOf(line, word, caseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive);
+		QEQUAL(indices.length(), result.length());
+		for (int i=0; i<indices.length(); i++) {
+			QEQUAL(indices[i], result[i]);
+		}
+	}
+
+	void test_indicesOf_RegExp_data() {
+		QTest::addColumn<QString>("line");
+		QTest::addColumn<QString>("rxString");
+		QTest::addColumn<QList<int> >("result");
+
+		QTest::newRow("noMatch") << "A test, a test, a test" << "foo" << (QList<int>());
+		QTest::newRow("multi1") << "A test, a test, a test" << "a\\s+t" << (QList<int>() << 8 << 16);
+		QTest::newRow("multi2") << "A test, a test, a test" << "[aA]\\s+t" << (QList<int>() << 0 << 8 << 16);
+		QTest::newRow("multi3") << "A test, a test, a test" << "test" << (QList<int>() << 2 << 10 << 18);
+	}
+
+	void test_indicesOf_RegExp() {
+		QFETCH(QString, line);
+		QFETCH(QString, rxString);
+		QFETCH(QList<int>, result);
+
+		QList<int> indices = indicesOf(line, QRegExp(rxString));
+		QEQUAL(indices.length(), result.length());
+		for (int i=0; i<indices.length(); i++) {
+			QEQUAL(indices[i], result[i]);
+		}
+	}
+
 };
 
 
