@@ -26,6 +26,7 @@
 #include "latexdocument.h"
 #include "latexeditorview.h"
 #include "latexcompleter.h"
+#include "findGlobalDialog.h"
 //#include "symbollistwidget.h"
 #include "symbolgridwidget.h"
 #include "xmltagslistwidget.h"
@@ -38,6 +39,7 @@
 #include "PDFDocument.h"
 #include <QSplashScreen>
 #include "session.h"
+#include "sessionlist.h"
 #include "qformatfactory.h"
 #include "qlanguagefactory.h"
 #include "qlinemarksinfocenter.h"
@@ -52,17 +54,18 @@ typedef QHash<QString,int> SymbolList;
 typedef QSet<QString> StringSet;
 
 enum SVNSTATUS {
-	Unknown=0,
-	Unmanaged,
-	Modified,
-	Locked,
-	CheckedIn,
-	InConflict
+	SVN_Unknown=0,
+	SVN_Unmanaged,
+	SVN_Modified,
+	SVN_Locked,
+	SVN_CheckedIn,
+	SVN_InConflict
 };
 
 class UserMenuDialog;
 class GrammarCheck;
 class FileSelector;
+class LatexReference;
 
 Q_DECLARE_METATYPE(QSet<QString>)
 
@@ -127,6 +130,7 @@ private:
 	QFormatScheme *m_formats, *m_formatsOldDefault;
 	QLanguageFactory* m_languages;
 	LatexCompleter* completer;
+	LatexReference* latexReference;
 	
 	QPointer<UnicodeInsertion> unicodeInsertionDialog;
 	
@@ -177,7 +181,8 @@ private:
 	SafeThread grammarCheckThread;
 	GrammarCheck *grammarCheck;
 	Bookmarks *bookmarks;
-	
+	SessionList *recentSessionList;
+
 	//dialogs
 	TextAnalysisDialog *textAnalysisDlg;
 	SpellerDialog *spellDlg;
@@ -210,7 +215,7 @@ protected slots:
 	void fileNewFromTemplate();
 	void fileMakeTemplate();
 	void fileOpen();
-	void fileRestoreSession(bool showProgress = true);
+	void fileRestoreSession(bool showProgress = true, bool warnMissing = true);
     void fileSave(const bool saveSilently=false);
 	void fileSaveAll();
 	void fileSaveAll(bool alsoUnnamedFiles, bool alwaysCurrentFile);
@@ -218,10 +223,19 @@ protected slots:
 private slots:
 	void fileSaveAs(const QString& fileName, const bool saveSilently);
 	void fileNewInternal(QString fileName="");
-protected slots:	
+protected slots:
+	void fileUtilCopyMove(bool move);
+	void fileUtilDelete();
+	void fileUtilRevert();
+	void fileUtilPermissions();
+	void fileUtilCopyFileName();
+	void fileUtilCopyMasterFileName();
 	void fileClose();
 	void fileCloseAll();
 	void fileExit();
+private:
+	void centerFileSelector();
+protected slots:
 	void fileOpenRecent();
 	void fileOpenAllRecent();
 	void fileRecentList();
@@ -234,7 +248,7 @@ protected slots:
 	void loadSession(const QString &fileName);
 	void fileSaveSession();
 private slots:
-	void restoreSession(const Session &s, bool showProgress = true);
+	void restoreSession(const Session &s, bool showProgress = true, bool warnMissing = true);
 	Session getCurrentSession();
 protected slots:
 	void MarkCurrentFileAsRecent();
@@ -265,6 +279,7 @@ private slots:
 	void fileDiffMerge();
 	void declareConflictResolved();
 protected slots:	
+    void openExternalFile();
 	void openExternalFile(const QString& name,const QString& defaultExt="tex",LatexDocument *doc=0); // signaled by latexViewer to open specific file
 	
 	void editUndo();
@@ -280,6 +295,10 @@ protected slots:
 	void editSectionPasteAfter(int line);
 	void editSectionPasteBefore();
 	void editSectionPasteBefore(int line);
+	void editTextToLowercase();
+	void editTextToUppercase();
+	void editTextToTitlecase();
+	void editFind();
 	void editPasteLatex();
 	void convertToLatex();
 	void editPasteRef();
@@ -299,9 +318,10 @@ protected slots:
 	void editChangeLineEnding();
 	void editSetupEncoding();
 	void editInsertUnicode();
-	void editInsertRefToNextLabel(bool backward=false);
-	void editInsertRefToPrevLabel();
+	void editInsertRefToNextLabel(const QString &refCmd="\\ref", bool backward=false);
+	void editInsertRefToPrevLabel(const QString &refCmd="\\ref");
 	void editFindGlobal();
+    void updateFindGlobal(int scope);
 	
 	void findWordRepetions();
 	void findNextWordRepetion();
@@ -334,8 +354,7 @@ protected slots:
 	void UpdateCaption();
 	void updateMasterDocumentCaption();
 	void updateUndoRedoStatus();
-	void editorTabChanged(int index);
-	void CloseEditorTab(int tab);
+	void currentEditorChanged();
 	void EditorTabMoved(int from,int to);
 	void editorAboutToChangeByTabClick(LatexEditorView *edFrom, LatexEditorView *edTo);
 
@@ -343,6 +362,7 @@ protected slots:
 	void ShowStructure();
 	void clickedOnStructureEntry(const QModelIndex & index);
 	void editRemovePlaceHolders();
+	void editRemoveCurrentPlaceHolder();
 
 	void NormalCompletion();
 	void InsertEnvironmentCompletion();
@@ -395,6 +415,8 @@ protected slots:
 	void runInternalPdfViewer(const QFileInfo& master, const QString& options);
 	void runBibliographyIfNecessary(const QFileInfo& cmd);
 
+    void searchExtendToggled(bool toggled);
+
 public slots:
 	void connectSubCommand(ProcessX* p, bool showStdoutLocallyDefault);
 private slots:	
@@ -406,7 +428,7 @@ private slots:
 	void endRunningCommand(const QString& commandMain, bool latex, bool pdf, bool async);
 	
 	
-	bool runCommand(const QString& commandline, QString* buffer = 0);
+	bool runCommand(const QString& commandline, QString* buffer = 0, QTextCodec *codecForBuffer = 0);
 protected slots:	
 	void processNotification(const QString& message);
 	void CleanAll();
@@ -477,7 +499,8 @@ protected slots:
 	void gotoLogEntryEditorOnly(int logEntryNumber);
 	bool gotoLogEntryAt(int newLineNumber);
 	bool gotoMark(bool backward, int id=-1);
-	void syncFromViewer(const QString &fileName, int line, bool activate, const QString& guessedWord);  //0 based, absolute file name, raise window
+	QList<int> findOccurencesApproximate(QString checkLine, const QString& guessedWord);
+	void syncFromViewer(const QString &fileName, int lineNr, bool activate, const QString& guessedWord);  //0 based, absolute file name, raise window
 	
 	void goBack();
 	void goForward();
@@ -532,6 +555,7 @@ protected slots:
 	void moveDocumentToEnd();
 	void moveDocumentToDest(int dest);
 	
+	void updateTexQNFA();
 	void updateHighlighting();
 	
 	void toggleGrammar(int type);
@@ -582,7 +606,7 @@ protected:
 	bool runBibliographyIfNecessaryEntered;
 	
 	LatexStyleParser *latexStyleParser;
-    KpathSeaParser *packageListReader;
+	PackageScanner *packageListReader;
 		
 	QMap<QString,QString> detectedEnvironmentsForHighlighting;
 	
@@ -599,6 +623,10 @@ protected:
 
     QSet<QString> latexPackageList,currentPackageList;
 
+    findGlobalDialog* findDlg;
+
+    QMap<QString,QString> *mReplacementList;
+
 public:
 	Q_PROPERTY(QString clipboard READ clipboardText WRITE setClipboardText);
 	Q_INVOKABLE QString clipboardText(const QClipboard::Mode& mode = QClipboard::Clipboard) const;
