@@ -1,18 +1,10 @@
 #include "utilsSystem.h"
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 #include <CoreFoundation/CFURL.h>
 #include <CoreFoundation/CFBundle.h>
 #endif
 
-int getSimplifiedSVNVersion(QString svnVersion) {
-	int cutoff = svnVersion.indexOf(QRegExp("[^0-9]"));
-	if (cutoff < 0)
-		return svnVersion.toInt();
-	else
-		return svnVersion.left(cutoff).toInt();
-}
-
 #ifdef Q_OS_WIN
 #include <windows.h>
 #endif
@@ -38,8 +30,16 @@ bool getDiskFreeSpace(const QString &path, quint64 &freeBytes) {
 #endif
 }
 
+QChar getPathListSeparator() {
+#ifdef Q_OS_WIN32
+	return QChar(';');
+#else
+	return QChar(':');
+#endif
+}
+
 QString getUserName() {
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	return QString(qgetenv("USERNAME"));
 #else
 	return QString(qgetenv("USER"));
@@ -47,7 +47,7 @@ QString getUserName() {
 }
 
 QString getUserDocumentFolder() {
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 	// typically "C:/Documents and Settings/Username/My Documents"
 	QSettings settings(QSettings::UserScope, "Microsoft", "Windows");
 	settings.beginGroup("CurrentVersion/Explorer/Shell Folders");
@@ -71,11 +71,10 @@ QStringList findResourceFiles(const QString& dirName, const QString& filter, QSt
 #define PREFIX ""
 #endif
 
-#if defined( Q_WS_X11 )
+#if defined( Q_WS_X11 ) || defined (Q_OS_LINUX)
 	searchFiles<<PREFIX"/share/texstudio"+dn; //X_11
-	searchFiles<<PREFIX"/share/texmakerx"+dn; //X_11
 #endif
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 	CFURLRef appUrlRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
 	CFStringRef macPath = CFURLCopyFileSystemPath(appUrlRef,
 												  kCFURLPOSIXPathStyle);
@@ -113,12 +112,11 @@ QString findResourceFile(const QString& fileName, bool allowOverride, QStringLis
 	foreach (const QString& s, additionalPreferredPaths)
 		if (s.endsWith('/') || s.endsWith('\\')) searchFiles << s;
 		else searchFiles << s + "/";
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 || defined Q_OS_LINUX || defined Q_OS_UNIX
 	searchFiles<<PREFIX"/share/texstudio/"; //X_11
-	searchFiles<<PREFIX"/share/texmakerx/"; //X_11
 	if (fileName.endsWith(".html")) searchFiles<<PREFIX"/share/doc/texstudio/html/";  //for Debian package
 #endif
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 	searchFiles<<QCoreApplication::applicationDirPath() + "/../Resources/"; //macx
 #endif
 	searchFiles<<QCoreApplication::applicationDirPath() + "/"; //windows old
@@ -214,7 +212,7 @@ QIcon getRealIconCached(const QString& icon){
 }
 
 bool isFileRealWritable(const QString& filename) {
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
 #if QT_VERSION >= 0x040700
     //bug in 4.7 still present in 4.8.0
     return (QFileInfo(filename).exists() && QFileInfo(filename).isWritable()) ||
@@ -223,15 +221,16 @@ bool isFileRealWritable(const QString& filename) {
     //thanks to Vistas virtual folders trying to open an unaccessable file can create it somewhere else
     return QFileInfo(filename).isWritable();
 #endif
+#else
+     QFile fi(filename);
+     bool result=false;
+     if (fi.exists()) result=fi.open(QIODevice::ReadWrite);
+     else {
+         result=fi.open(QIODevice::WriteOnly);
+         fi.remove();
+     }
+     return result;
 #endif
-    QFile fi(filename);
-    bool result=false;
-    if (fi.exists()) result=fi.open(QIODevice::ReadWrite);
-    else {
-        result=fi.open(QIODevice::WriteOnly);
-        fi.remove();
-    }
-    return result;
 }
 
 bool isExistingFileRealWritable(const QString& filename) {
@@ -241,7 +240,7 @@ bool isExistingFileRealWritable(const QString& filename) {
 QString ensureTrailingDirSeparator(const QString& dirPath){
     if (dirPath.endsWith("/")) return dirPath;
     if (dirPath.endsWith(QDir::separator())) return dirPath;
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN32
     if (dirPath.endsWith("\\")) return dirPath; //you can create a directory named \ on linux
 #endif
     return dirPath+"/";
@@ -309,6 +308,22 @@ QString getPathfromFilename(const QString &compFile){
     return dir;
 }
 
+QString findAbsoluteFilePath(const QString & relName, const QString &extension, const QStringList &searchPaths, const QString& fallbackPath) {
+	QString s=relName;
+	QString ext = extension;
+	if (!ext.isEmpty() && !ext.startsWith(".")) ext = "." + ext;
+	if (!s.endsWith(ext,Qt::CaseInsensitive)) s+=ext;
+	QFileInfo fi(s);
+	if (!fi.isRelative()) return s;
+	foreach (const QString &path, searchPaths) {
+		fi.setFile(QDir(path), s);
+		if (fi.exists()) return fi.absoluteFilePath();
+	}
+	QString fbp = fallbackPath;
+	if (!fbp.isEmpty() && !fbp.endsWith('/') && !fbp.endsWith(QDir::separator())) fbp += QDir::separator();
+	return fbp + s; // fallback
+}
+
 int x11desktop_env() {
     // 0 : no kde ; 3: kde ; 4 : kde4 ;
     QString kdesession= ::getenv("KDE_FULL_SESSION");
@@ -338,8 +353,9 @@ bool isRetinaMac() {
             isRetina = true;
     }
     return isRetina;
+#else
+     return false;
 #endif
-    return false;
 }
 
 bool hasAtLeastQt(int major, int minor){
@@ -360,6 +376,19 @@ bool connectUnique(const QObject * sender, const char * signal, const QObject *
 #endif
 }
 
+// run the command in a separate process, wait and return the result
+// use for internal queries that should be silent. Not to be mixed up with BuildManager::runCommand
+QString execCommand(const QString & cmd) {
+	if(cmd.isEmpty()) return QString();
+	QProcess myProc(0);
+	myProc.start(cmd);
+	myProc.waitForFinished();
+	QString result;
+	if(myProc.exitCode() == 0) {
+		result=myProc.readAllStandardOutput();
+	}
+	return result.trimmed();
+}
 
 void ThreadBreaker::sleep(int s){
     QThread::sleep(s);
